@@ -23,15 +23,14 @@
 #include "agentbase.h"
 #include "agentbase_p.h"
 
+#include "changerecorder.h"
 #include "controladaptor.h"
-#include "statusadaptor.h"
+#include "itemfetchjob.h"
 #include "monitor_p.h"
-#include "xdgbasedirs_p.h"
-
 #include "session.h"
 #include "session_p.h"
-#include "changerecorder.h"
-#include "itemfetchjob.h"
+#include "statusadaptor.h"
+#include "xdgbasedirs_p.h"
 
 #include <kaboutdata.h>
 #include <kcmdlineargs.h>
@@ -42,8 +41,8 @@
 #include <QtCore/QDir>
 #include <QtCore/QSettings>
 #include <QtCore/QTimer>
-#include <QtGui/QApplication>
 #include <QtDBus/QtDBus>
+#include <QtGui/QApplication>
 
 #include <signal.h>
 #include <stdlib.h>
@@ -62,7 +61,6 @@ AgentBase::Observer::~Observer()
 
 void AgentBase::Observer::itemAdded( const Item &item, const Collection &collection )
 {
-  kDebug() << "sAgentBase=" << (void*) sAgentBase << "this=" << (void*) this;
   Q_UNUSED( item );
   Q_UNUSED( collection );
   if ( sAgentBase != 0 )
@@ -71,7 +69,6 @@ void AgentBase::Observer::itemAdded( const Item &item, const Collection &collect
 
 void AgentBase::Observer::itemChanged( const Item &item, const QSet<QByteArray> &partIdentifiers )
 {
-  kDebug() << "sAgentBase=" << (void*) sAgentBase << "this=" << (void*) this;
   Q_UNUSED( item );
   Q_UNUSED( partIdentifiers );
   if ( sAgentBase != 0 )
@@ -80,7 +77,6 @@ void AgentBase::Observer::itemChanged( const Item &item, const QSet<QByteArray>
 
 void AgentBase::Observer::itemRemoved( const Item &item )
 {
-  kDebug() << "sAgentBase=" << (void*) sAgentBase << "this=" << (void*) this;
   Q_UNUSED( item );
   if ( sAgentBase != 0 )
     sAgentBase->d_ptr->changeProcessed();
@@ -88,7 +84,6 @@ void AgentBase::Observer::itemRemoved( const Item &item )
 
 void AgentBase::Observer::collectionAdded( const Akonadi::Collection &collection, const Akonadi::Collection &parent )
 {
-  kDebug() << "sAgentBase=" << (void*) sAgentBase << "this=" << (void*) this;
   Q_UNUSED( collection );
   Q_UNUSED( parent );
   if ( sAgentBase != 0 )
@@ -97,7 +92,6 @@ void AgentBase::Observer::collectionAdded( const Akonadi::Collection &collection
 
 void AgentBase::Observer::collectionChanged( const Collection &collection )
 {
-  kDebug() << "sAgentBase=" << (void*) sAgentBase << "this=" << (void*) this;
   Q_UNUSED( collection );
   if ( sAgentBase != 0 )
     sAgentBase->d_ptr->changeProcessed();
@@ -105,12 +99,51 @@ void AgentBase::Observer::collectionChanged( const Collection &collection )
 
 void AgentBase::Observer::collectionRemoved( const Collection &collection )
 {
-  kDebug() << "sAgentBase=" << (void*) sAgentBase << "this=" << (void*) this;
   Q_UNUSED( collection );
   if ( sAgentBase != 0 )
     sAgentBase->d_ptr->changeProcessed();
 }
 
+void AgentBase::ObserverV2::itemMoved( const Akonadi::Item &item, const Akonadi::Collection &source, const Akonadi::Collection &dest )
+{
+  Q_UNUSED( item );
+  Q_UNUSED( source );
+  Q_UNUSED( dest );
+  if ( sAgentBase != 0 )
+    sAgentBase->d_ptr->changeProcessed();
+}
+
+void AgentBase::ObserverV2::itemLinked( const Akonadi::Item& item, const Akonadi::Collection& collection )
+{
+  Q_UNUSED( item );
+  Q_UNUSED( collection );
+  if ( sAgentBase != 0 )
+    sAgentBase->d_ptr->changeProcessed();
+}
+
+void AgentBase::ObserverV2::itemUnlinked( const Akonadi::Item& item, const Akonadi::Collection& collection )
+{
+  Q_UNUSED( item );
+  Q_UNUSED( collection );
+  if ( sAgentBase != 0 )
+    sAgentBase->d_ptr->changeProcessed();
+}
+
+void AgentBase::ObserverV2::collectionMoved( const Akonadi::Collection &collection, const Akonadi::Collection &source, const Akonadi::Collection &dest )
+{
+  Q_UNUSED( collection );
+  Q_UNUSED( source );
+  Q_UNUSED( dest );
+  if ( sAgentBase != 0 )
+    sAgentBase->d_ptr->changeProcessed();
+}
+
+void AgentBase::ObserverV2::collectionChanged( const Akonadi::Collection &collection, const QSet<QByteArray> &partIdentifiers )
+{
+  Q_UNUSED( partIdentifiers );
+  collectionChanged( collection );
+}
+
 //@cond PRIVATE
 
 AgentBasePrivate::AgentBasePrivate( AgentBase *parent )
@@ -156,28 +189,54 @@ void AgentBasePrivate::init()
 
   mOnline = mSettings->value( QLatin1String( "Agent/Online" ), true ).toBool();
 
+  // reinitialize the status message now that online state is available
+  mStatusMessage = defaultReadyMessage();
+
+  mName = mSettings->value( QLatin1String( "Agent/Name" ) ).toString();
+  if ( mName.isEmpty() ) {
+    mName = mSettings->value( QLatin1String( "Resource/Name" ) ).toString();
+    if ( !mName.isEmpty() ) {
+      mSettings->remove( QLatin1String( "Resource/Name" ) );
+      mSettings->setValue( QLatin1String( "Agent/Name" ), mName );
+    }
+  }
+
   connect( mMonitor, SIGNAL( itemAdded( const Akonadi::Item&, const Akonadi::Collection& ) ),
            SLOT( itemAdded( const Akonadi::Item&, const Akonadi::Collection& ) ) );
   connect( mMonitor, SIGNAL( itemChanged( const Akonadi::Item&, const QSet<QByteArray>& ) ),
            SLOT( itemChanged( const Akonadi::Item&, const QSet<QByteArray>& ) ) );
+  connect( mMonitor, SIGNAL( itemMoved( const Akonadi::Item&, const Akonadi::Collection&, const Akonadi::Collection& ) ),
+           SLOT( itemMoved( const Akonadi::Item&, const Akonadi::Collection&, const Akonadi::Collection& ) ) );
   connect( mMonitor, SIGNAL( itemRemoved( const Akonadi::Item& ) ),
            SLOT( itemRemoved( const Akonadi::Item& ) ) );
-  connect( mMonitor, SIGNAL(collectionAdded(Akonadi::Collection,Akonadi::Collection)),
-           SLOT(collectionAdded(Akonadi::Collection,Akonadi::Collection)) );
+  connect( mMonitor, SIGNAL( collectionAdded( const Akonadi::Collection&, const Akonadi::Collection& ) ),
+           SLOT( collectionAdded( const Akonadi::Collection&, const Akonadi::Collection& ) ) );
+  connect( mMonitor, SIGNAL( itemLinked( const Akonadi::Item&, const Akonadi::Collection& ) ),
+           SLOT( itemLinked( const Akonadi::Item&, const Akonadi::Collection& ) ) );
+  connect( mMonitor, SIGNAL( itemUnlinked( const Akonadi::Item&, const Akonadi::Collection& ) ),
+           SLOT( itemUnlinked( const Akonadi::Item&, const Akonadi::Collection& ) ) );
   connect( mMonitor, SIGNAL( collectionChanged( const Akonadi::Collection& ) ),
            SLOT( collectionChanged( const Akonadi::Collection& ) ) );
+  connect( mMonitor, SIGNAL( collectionChanged( const Akonadi::Collection&, const QSet<QByteArray>& ) ),
+           SLOT( collectionChanged( const Akonadi::Collection&, const QSet<QByteArray>& ) ) );
+  connect( mMonitor, SIGNAL( collectionMoved( const Akonadi::Collection&, const Akonadi::Collection&, const Akonadi::Collection& ) ),
+           SLOT( collectionMoved( const Akonadi::Collection&, const Akonadi::Collection&, const Akonadi::Collection& ) ) );
   connect( mMonitor, SIGNAL( collectionRemoved( const Akonadi::Collection& ) ),
            SLOT( collectionRemoved( const Akonadi::Collection& ) ) );
 
-  connect( q, SIGNAL( status( int, QString ) ), q, SLOT( slotStatus( int, QString ) ) );
+  connect( q, SIGNAL( status( int, const QString& ) ), q, SLOT( slotStatus( int, const QString& ) ) );
   connect( q, SIGNAL( percent( int ) ), q, SLOT( slotPercent( int ) ) );
-  connect( q, SIGNAL( warning( QString ) ), q, SLOT( slotWarning( QString ) ) );
-  connect( q, SIGNAL( error( QString ) ), q, SLOT( slotError( QString ) ) );
+  connect( q, SIGNAL( warning( const QString& ) ), q, SLOT( slotWarning( const QString& ) ) );
+  connect( q, SIGNAL( error( const QString& ) ), q, SLOT( slotError( const QString& ) ) );
 
   // Use reference counting to allow agents to finish internal jobs when the
   // agent is stopped.
   KGlobal::ref();
-  KGlobal::setAllowQuit(true);
+  KGlobal::setAllowQuit( true );
+
+  // disable session management
+  if ( KApplication::kApplication() )
+    KApplication::kApplication()->disableSessionManagement();
 
   QTimer::singleShot( 0, q, SLOT( delayedInit() ) );
 }
@@ -192,42 +251,124 @@ void AgentBasePrivate::delayedInit()
 
 void AgentBasePrivate::itemAdded( const Akonadi::Item &item, const Akonadi::Collection &collection )
 {
-  kDebug() << "mObserver=" << (void*) mObserver << "this=" << (void*) this;
   if ( mObserver != 0 )
     mObserver->itemAdded( item, collection );
 }
 
 void AgentBasePrivate::itemChanged( const Akonadi::Item &item, const QSet<QByteArray> &partIdentifiers )
 {
-  kDebug() << "mObserver=" << (void*) mObserver << "this=" << (void*) this;
   if ( mObserver != 0 )
     mObserver->itemChanged( item, partIdentifiers );
 }
 
+void AgentBasePrivate::itemMoved( const Akonadi::Item &item, const Akonadi::Collection &source, const Akonadi::Collection &dest )
+{
+  AgentBase::ObserverV2 *observer2 = dynamic_cast<AgentBase::ObserverV2*>( mObserver );
+  if ( mObserver ) {
+    // inter-resource moves, requires we know which resources the source and destination are in though
+    if ( !source.resource().isEmpty() && !dest.resource().isEmpty() ) {
+      if ( source.resource() != dest.resource() ) {
+        if ( source.resource() == q_ptr->identifier() ) // moved away from us
+          mObserver->itemRemoved( item );
+        else if ( dest.resource() == q_ptr->identifier() ) // moved to us
+          mObserver->itemAdded( item, dest );
+        else if ( observer2 )
+          observer2->itemMoved( item, source, dest );
+        else
+          // not for us, not sure if we should get here at all
+          changeProcessed();
+        return;
+      }
+    }
+    // intra-resource move
+    if ( observer2 ) {
+      observer2->itemMoved( item, source, dest );
+    } else {
+      // ### we cannot just call itemRemoved here as this will already trigger changeProcessed()
+      // so, just itemAdded() is good enough as no resource can have implemented intra-resource moves anyway
+      // without using ObserverV2
+      mObserver->itemAdded( item, dest );
+      // mObserver->itemRemoved( item );
+    }
+  }
+}
+
 void AgentBasePrivate::itemRemoved( const Akonadi::Item &item )
 {
-  kDebug() << "mObserver=" << (void*) mObserver << "this=" << (void*) this;
   if ( mObserver != 0 )
     mObserver->itemRemoved( item );
 }
 
+void AgentBasePrivate::itemLinked( const Akonadi::Item &item, const Akonadi::Collection &collection )
+{
+  AgentBase::ObserverV2 *observer2 = dynamic_cast<AgentBase::ObserverV2*>( mObserver );
+  if ( observer2 )
+    observer2->itemLinked( item, collection );
+  else
+    changeProcessed();
+}
+
+void AgentBasePrivate::itemUnlinked( const Akonadi::Item &item, const Akonadi::Collection &collection )
+{
+  AgentBase::ObserverV2 *observer2 = dynamic_cast<AgentBase::ObserverV2*>( mObserver );
+  if ( observer2 )
+    observer2->itemUnlinked( item, collection );
+  else
+    changeProcessed();
+}
+
 void AgentBasePrivate::collectionAdded( const Akonadi::Collection &collection, const Akonadi::Collection &parent )
 {
-  kDebug() << "mObserver=" << (void*) mObserver << "this=" << (void*) this;
   if ( mObserver != 0 )
     mObserver->collectionAdded( collection, parent );
 }
 
 void AgentBasePrivate::collectionChanged( const Akonadi::Collection &collection )
 {
-  kDebug() << "mObserver=" << (void*) mObserver << "this=" << (void*) this;
-  if ( mObserver != 0 )
+  AgentBase::ObserverV2 *observer2 = dynamic_cast<AgentBase::ObserverV2*>( mObserver );
+  if ( mObserver != 0 && observer2 == 0 ) // For ObserverV2 we use the variant with the part identifiers
     mObserver->collectionChanged( collection );
 }
 
+void AgentBasePrivate::collectionChanged( const Akonadi::Collection &collection, const QSet<QByteArray> &partIdentifiers )
+{
+  AgentBase::ObserverV2 *observer2 = dynamic_cast<AgentBase::ObserverV2*>( mObserver );
+  if ( observer2 != 0 )
+    observer2->collectionChanged( collection, partIdentifiers );
+}
+
+void AgentBasePrivate::collectionMoved( const Akonadi::Collection &collection, const Akonadi::Collection &source, const Akonadi::Collection &dest )
+{
+  AgentBase::ObserverV2 *observer2 = dynamic_cast<AgentBase::ObserverV2*>( mObserver );
+  if ( mObserver ) {
+    // inter-resource moves, requires we know which resources the source and destination are in though
+    if ( !source.resource().isEmpty() && !dest.resource().isEmpty() ) {
+      if ( source.resource() != dest.resource() ) {
+        if ( source.resource() == q_ptr->identifier() ) // moved away from us
+          mObserver->collectionRemoved( collection );
+        else if ( dest.resource() == q_ptr->identifier() ) // moved to us
+          mObserver->collectionAdded( collection, dest );
+        else if ( observer2 )
+          observer2->collectionMoved( collection, source, dest );
+        else // not for us, not sure if we should get here at all
+          changeProcessed();
+        return;
+      }
+    }
+    // intra-resource move
+    if ( observer2 ) {
+      observer2->collectionMoved( collection, source, dest );
+    } else {
+      // ### we cannot just call collectionRemoved here as this will already trigger changeProcessed()
+      // so, just collectionAdded() is good enough as no resource can have implemented intra-resource moves anyway
+      // without using ObserverV2
+      mObserver->collectionAdded( collection, dest );
+    }
+  }
+}
+
 void AgentBasePrivate::collectionRemoved( const Akonadi::Collection &collection )
 {
-  kDebug() << "mObserver=" << (void*) mObserver << "this=" << (void*) this;
   if ( mObserver != 0 )
     mObserver->collectionRemoved( collection );
 }
@@ -296,8 +437,6 @@ AgentBase::AgentBase( const QString & id )
   sAgentBase = this;
   d_ptr->mId = id;
   d_ptr->init();
-  if ( KApplication::kApplication() )
-    KApplication::kApplication()->disableSessionManagement();
 }
 
 AgentBase::AgentBase( AgentBasePrivate* d, const QString &id ) :
@@ -313,13 +452,11 @@ AgentBase::~AgentBase()
   delete d_ptr;
 }
 
-static char* sAgentAppName = 0;
-
 QString AgentBase::parseArguments( int argc, char **argv )
 {
   QString identifier;
   if ( argc < 3 ) {
-    kDebug( 5250 ) << "Not enough arguments passed...";
+    kDebug() << "Not enough arguments passed...";
     exit( 1 );
   }
 
@@ -329,16 +466,22 @@ QString AgentBase::parseArguments( int argc, char **argv )
   }
 
   if ( identifier.isEmpty() ) {
-    kDebug( 5250 ) << "Identifier argument missing";
+    kDebug() << "Identifier argument missing";
     exit( 1 );
   }
 
-  sAgentAppName = qstrdup( identifier.toLatin1().constData() );
-  KCmdLineArgs::init( argc, argv, sAgentAppName, 0, ki18n("Akonadi Agent"),"0.1" ,
-                      ki18n("Akonadi Agent") );
+  QByteArray catalog;
+  char *p = strrchr( argv[0], '/' );
+  if ( p )
+    catalog = QByteArray( p + 1 );
+  else
+    catalog = QByteArray( argv[0] );
+
+  KCmdLineArgs::init( argc, argv, identifier.toLatin1(), catalog, ki18n( "Akonadi Agent" ), "0.1",
+                      ki18n( "Akonadi Agent" ) );
 
   KCmdLineOptions options;
-  options.add("identifier <argument>", ki18n("Agent identifier"));
+  options.add( "identifier <argument>", ki18n( "Agent identifier" ) );
   KCmdLineArgs::addCmdLineOptions( options );
 
   return identifier;
@@ -349,10 +492,9 @@ QString AgentBase::parseArguments( int argc, char **argv )
 int AgentBase::init( AgentBase *r )
 {
   QApplication::setQuitOnLastWindowClosed( false );
-  KGlobal::locale()->insertCatalog( "libakonadi" );
+  KGlobal::locale()->insertCatalog( QLatin1String( "libakonadi" ) );
   int rv = kapp->exec();
   delete r;
-  delete[] sAgentAppName;
   return rv;
 }
 
@@ -399,7 +541,7 @@ void AgentBase::setNeedsNetwork( bool needsNetwork )
   if ( d->mNeedsNetwork ) {
     connect( Solid::Networking::notifier()
            , SIGNAL( statusChanged( Solid::Networking::Status ) )
-           , d, SLOT( slotNetworkStatusChange( Solid::Networking::Status ) ) );
+           , this, SLOT( slotNetworkStatusChange( Solid::Networking::Status ) ) );
   } else {
     disconnect( Solid::Networking::notifier(), 0, 0, 0 );
     setOnline( true );
@@ -423,9 +565,10 @@ void AgentBase::doSetOnline( bool online )
 void AgentBase::configure( WId windowId )
 {
   Q_UNUSED( windowId );
+  emit configurationDialogAccepted();
 }
 
-#ifdef Q_OS_WIN
+#ifdef Q_OS_WIN //krazy:exclude=cpp
 void AgentBase::configure( qlonglong windowId )
 {
   configure( reinterpret_cast<WId>( windowId ) );
@@ -434,18 +577,19 @@ void AgentBase::configure( qlonglong windowId )
 
 WId AgentBase::winIdForDialogs() const
 {
-  bool registered = QDBusConnection::sessionBus().interface()->isServiceRegistered( QLatin1String("org.freedesktop.akonaditray") );
+  const bool registered = QDBusConnection::sessionBus().interface()->isServiceRegistered( QLatin1String( "org.freedesktop.akonaditray" ) );
   if ( !registered )
     return 0;
 
-  QDBusInterface dbus( QLatin1String("org.freedesktop.akonaditray"), QLatin1String("/Actions"),
-                       QLatin1String("org.freedesktop.Akonadi.Tray") );
-  QDBusMessage reply = dbus.call( QLatin1String("getWinId") );
+  QDBusInterface dbus( QLatin1String( "org.freedesktop.akonaditray" ), QLatin1String( "/Actions" ),
+                       QLatin1String( "org.freedesktop.Akonadi.Tray" ) );
+  const QDBusMessage reply = dbus.call( QLatin1String( "getWinId" ) );
 
   if ( reply.type() == QDBusMessage::ErrorMessage )
     return 0;
 
-  WId winid = (WId)reply.arguments().at( 0 ).toLongLong();
+  const WId winid = (WId)reply.arguments().at( 0 ).toLongLong();
+
   return winid;
 }
 
@@ -469,6 +613,12 @@ void AgentBase::aboutToQuit()
 void AgentBase::cleanup()
 {
   Q_D( AgentBase );
+  // prevent the monitor from picking up deletion signals for our own data if we are a resource
+  // and thus avoid that we kill our own data as last act before our own death
+  d->mMonitor->blockSignals( true );
+
+  aboutToQuit();
+
   const QString fileName = d->mSettings->fileName();
 
   /*
@@ -489,12 +639,11 @@ void AgentBase::cleanup()
   QString configFile = KStandardDirs::locateLocal( "config", KGlobal::config()->name() );
   QFile::remove( configFile );
 
-  QCoreApplication::quit();
+  KGlobal::deref();
 }
 
 void AgentBase::registerObserver( Observer *observer )
 {
-  kDebug() << "observer=" << (void*) observer << "this=" << (void*) this;
   d_ptr->mObserver = observer;
 }
 
@@ -503,6 +652,35 @@ QString AgentBase::identifier() const
   return d_ptr->mId;
 }
 
+void AgentBase::setAgentName( const QString &name )
+{
+  Q_D( AgentBase );
+  if ( name == d->mName )
+    return;
+
+  // TODO: rename collection
+  d->mName = name;
+
+  if ( d->mName.isEmpty() || d->mName == d->mId ) {
+    d->mSettings->remove( QLatin1String( "Resource/Name" ) );
+    d->mSettings->remove( QLatin1String( "Agent/Name" ) );
+  } else
+    d->mSettings->setValue( QLatin1String( "Agent/Name" ), d->mName );
+
+  d->mSettings->sync();
+
+  emit agentNameChanged( d->mName );
+}
+
+QString AgentBase::agentName() const
+{
+  Q_D( const AgentBase );
+  if ( d->mName.isEmpty() )
+    return d->mId;
+  else
+    return d->mName;
+}
+
 void AgentBase::changeProcessed()
 {
   Q_D( AgentBase );
@@ -514,6 +692,11 @@ ChangeRecorder * AgentBase::changeRecorder() const
   return d_ptr->mMonitor;
 }
 
+void AgentBase::abort()
+{
+  emit abortRequested();
+}
+
 void AgentBase::reconfigure()
 {
   emit reloadConfiguration();
