@@ -23,22 +23,23 @@
 #include <gpgme++/config-gpgme++.h>
 #include <gpgme++/verificationresult.h>
 #include <gpgme++/notation.h>
-#include "shared.h"
 #include "result_p.h"
 #include "util.h"
 
 #include <gpgme.h>
 
+#include <istream>
 #include <algorithm>
+#include <iterator>
 #include <string>
 #include <cstring>
 #include <cstdlib>
 
 #include <string.h>
 
-class GpgME::VerificationResult::Private : public GpgME::Shared {
+class GpgME::VerificationResult::Private {
 public:
-  Private( const gpgme_verify_result_t r ) : Shared() {
+  explicit Private( const gpgme_verify_result_t r ) {
     if ( !r )
       return;
 #ifdef HAVE_GPGME_VERIFY_RESULT_T_FILE_NAME
@@ -51,6 +52,15 @@ public:
       gpgme_signature_t scopy = new _gpgme_signature( *is );
       if ( is->fpr )
 	scopy->fpr = strdup( is->fpr );
+#ifdef HAVE_GPGME_SIGNATURE_T_PKA_FIELDS
+// PENDING(marc) why does this crash on Windows in strdup()?
+# ifndef _WIN32
+     if ( is->pka_address )
+	scopy->pka_address = strdup( is->pka_address );
+# else
+     scopy->pka_address = 0;
+# endif
+#endif
       scopy->next = 0;
       sigs.push_back( scopy );
       // copy notations:
@@ -77,6 +87,9 @@ public:
   ~Private() {
     for ( std::vector<gpgme_signature_t>::iterator it = sigs.begin() ; it != sigs.end() ; ++it ) {
       std::free( (*it)->fpr );
+#ifdef HAVE_GPGME_SIGNATURE_T_PKA_FIELDS
+      std::free( (*it)->pka_address );
+#endif
       delete *it; *it = 0;
     }
     for ( std::vector< std::vector<Nota> >::iterator it = nota.begin() ; it != nota.end() ; ++it )
@@ -102,15 +115,24 @@ public:
 };
 
 GpgME::VerificationResult::VerificationResult( gpgme_ctx_t ctx, int error )
-  : GpgME::Result( error ), d( 0 )
+  : GpgME::Result( error ), d()
 {
-  if ( error || !ctx )
+  init( ctx );
+}
+
+GpgME::VerificationResult::VerificationResult( gpgme_ctx_t ctx, const Error & error )
+  : GpgME::Result( error ), d()
+{
+  init( ctx );
+}
+
+void GpgME::VerificationResult::init( gpgme_ctx_t ctx ) {
+  if ( !ctx )
     return;
   gpgme_verify_result_t res = gpgme_op_verify_result( ctx );
   if ( !res )
     return;
-  d = new Private( res );
-  d->ref();
+  d.reset( new Private( res ) );
 }
 
 make_standard_stuff(VerificationResult)
@@ -140,26 +162,13 @@ std::vector<GpgME::Signature> GpgME::VerificationResult::signatures() const {
 
 
 
-GpgME::Signature::Signature( VerificationResult::Private * parent, unsigned int i )
+GpgME::Signature::Signature( const boost::shared_ptr<VerificationResult::Private> & parent, unsigned int i )
   : d( parent ), idx( i )
 {
-  if ( d )
-    d->ref();
+
 }
 
-GpgME::Signature::Signature() : d( 0 ), idx( 0 ) {}
-
-GpgME::Signature::Signature( const Signature & other )
-  : d( other.d ), idx( other.idx )
-{
-  if ( d )
-    d->ref();
-}
-
-GpgME::Signature::~Signature() {
-  if ( d )
-    d->unref();
-}
+GpgME::Signature::Signature() : d(), idx( 0 ) {}
 
 bool GpgME::Signature::isNull() const {
   return !d || idx >= d->sigs.size() ;
@@ -319,15 +328,14 @@ std::vector<GpgME::Notation> GpgME::Signature::notations() const {
 
 class GpgME::Notation::Private {
 public:
-    Private() : d( 0 ), sidx( 0 ), nidx( 0 ), nota( 0 ) {}
-    Private( VerificationResult::Private * priv, unsigned int sindex, unsigned int nindex )
+    Private() : d(), sidx( 0 ), nidx( 0 ), nota( 0 ) {}
+    Private( const boost::shared_ptr<VerificationResult::Private> & priv, unsigned int sindex, unsigned int nindex )
 	: d( priv ), sidx( sindex ), nidx( nindex ), nota( 0 )
     {
-      if ( d )
-        d->ref();
+
     }
     Private( gpgme_sig_notation_t n )
-	: d( 0 ), sidx( 0 ), nidx( 0 ), nota( n ? new _gpgme_sig_notation( *n ) : 0 )
+	: d(), sidx( 0 ), nidx( 0 ), nota( n ? new _gpgme_sig_notation( *n ) : 0 )
     {
       if ( nota && nota->name )
         nota->name = strdup( nota->name );
@@ -337,29 +345,26 @@ public:
     Private( const Private & other )
 	: d( other.d ), sidx( other.sidx ), nidx( other.nidx ), nota( other.nota )
     {
-      if ( d )
-        d->ref();
       if ( nota ) {
         nota->name = strdup( nota->name );
         nota->value = strdup( nota->value );
       }
     }
     ~Private() {
-      if ( d )
-        d->unref();
       if ( nota ) {
         std::free( nota->name );  nota->name = 0;
         std::free( nota->value ); nota->value = 0;
+        delete nota;
       }
     }
 
-    VerificationResult::Private * d;
+    boost::shared_ptr<VerificationResult::Private> d;
     unsigned int sidx, nidx;
     gpgme_sig_notation_t nota;
 };
 
 
-GpgME::Notation::Notation( VerificationResult::Private * parent, unsigned int sindex, unsigned int nindex )
+GpgME::Notation::Notation( const boost::shared_ptr<VerificationResult::Private> & parent, unsigned int sindex, unsigned int nindex )
   : d( new Private( parent, sindex, nindex ) )
 {
 
@@ -371,17 +376,7 @@ GpgME::Notation::Notation( gpgme_sig_notation_t nota )
 
 }
 
-GpgME::Notation::Notation() : d( 0 ) {}
-
-GpgME::Notation::Notation( const Notation & other )
-  : d( other.d ? new Private( *other.d ) : 0 )
-{
-
-}
-
-GpgME::Notation::~Notation() {
-  delete d; d = 0;
-}
+GpgME::Notation::Notation() : d() {}
 
 bool GpgME::Notation::isNull() const {
     if ( !d )
@@ -410,7 +405,7 @@ GpgME::Notation::Flags GpgME::Notation::flags() const {
     return
       convert_from_gpgme_sig_notation_flags_t(
 #ifdef HAVE_GPGME_SIG_NOTATION_FLAGS_T
-        isNull() ? 0 : 
+        isNull() ? 0 :
         d->d ? d->d->nota[d->sidx][d->nidx].flags :
         d->nota ? d->nota->flags : 0 );
 #else
@@ -427,3 +422,88 @@ bool GpgME::Notation::isCritical() const {
 }
 
 
+
+std::ostream & GpgME::operator<<( std::ostream & os, const VerificationResult & result ) {
+    os << "GpgME::VerificationResult(";
+    if ( !result.isNull() ) {
+        os << "\n error:      " << result.error()
+           << "\n fileName:   " << protect( result.fileName() )
+           << "\n signatures:\n";
+        const std::vector<Signature> sigs = result.signatures();
+        std::copy( sigs.begin(), sigs.end(),
+                   std::ostream_iterator<Signature>( os, "\n" ) );
+    }
+    return os << ')';
+}
+
+std::ostream & GpgME::operator<<( std::ostream & os, Signature::PKAStatus pkaStatus ) {
+#define OUTPUT( x ) if ( !(pkaStatus & (GpgME::Signature:: x)) ) {} else do { os << #x " "; } while(0)
+    os << "GpgME::Signature::PKAStatus(";
+    OUTPUT( UnknownPKAStatus );
+    OUTPUT( PKAVerificationFailed );
+    OUTPUT( PKAVerificationSucceeded );
+#undef OUTPUT
+    return os << ')';
+}
+
+std::ostream & GpgME::operator<<( std::ostream & os, Signature::Summary summary ) {
+#define OUTPUT( x ) if ( !(summary & (GpgME::Signature:: x)) ) {} else do { os << #x " "; } while(0)
+    os << "GpgME::Signature::Summary(";
+    OUTPUT( Valid );
+    OUTPUT( Green );
+    OUTPUT( Red );
+    OUTPUT( KeyRevoked );
+    OUTPUT( KeyExpired );
+    OUTPUT( SigExpired );
+    OUTPUT( KeyMissing );
+    OUTPUT( CrlMissing );
+    OUTPUT( CrlTooOld );
+    OUTPUT( BadPolicy );
+    OUTPUT( SysError );
+#undef OUTPUT
+    return os << ')';
+}
+
+std::ostream & GpgME::operator<<( std::ostream & os, const Signature & sig ) {
+    os << "GpgME::Signature(";
+    if ( !sig.isNull() ) {
+        os << "\n Summary:                   " << sig.summary()
+           << "\n Fingerprint:               " << protect( sig.fingerprint() )
+           << "\n Status:                    " << sig.status()
+           << "\n creationTime:              " << sig.creationTime()
+           << "\n expirationTime:            " << sig.expirationTime()
+           << "\n isWrongKeyUsage:           " << sig.isWrongKeyUsage()
+           << "\n isVerifiedUsingChainModel: " << sig.isVerifiedUsingChainModel()
+           << "\n pkaStatus:                 " << sig.pkaStatus()
+           << "\n pkaAddress:                " << protect( sig.pkaAddress() )
+           << "\n validity:                  " << sig.validityAsString()
+           << "\n nonValidityReason:         " << sig.nonValidityReason()
+           << "\n publicKeyAlgorithm:        " << protect( sig.publicKeyAlgorithmAsString() )
+           << "\n hashAlgorithm:             " << protect( sig.hashAlgorithmAsString() )
+           << "\n policyURL:                 " << protect( sig.policyURL() )
+           << "\n notations:\n";
+        const std::vector<Notation> nota = sig.notations();
+        std::copy( nota.begin(), nota.end(),
+                   std::ostream_iterator<Notation>( os, "\n" ) );
+    }
+    return os << ')';
+}
+
+std::ostream & GpgME::operator<<( std::ostream & os, Notation::Flags flags ) {
+    os << "GpgME::Notation::Flags(";
+#define OUTPUT( x ) if ( !(flags & (GpgME::Notation:: x)) ) {} else do { os << #x " "; } while(0)
+    OUTPUT( HumanReadable );
+    OUTPUT( Critical );
+#undef OUTPUT
+    return os << ')';
+}
+
+std::ostream & GpgME::operator<<( std::ostream & os, const Notation & nota ) {
+    os << "GpgME::Signature::Notation(";
+    if ( !nota.isNull() )
+        os << "\n name:  " << protect( nota.name() )
+           << "\n value: " << protect( nota.value() )
+           << "\n flags: " << nota.flags()
+           << '\n';
+    return os << ")";
+}
