@@ -29,8 +29,7 @@
 #include <kstandarddirs.h>
 #include <kdialog.h>
 
-#include <QtGui/QLabel>
-#include <QtGui/QLayout>
+#include <QtGui/QFormLayout>
 
 #include <unistd.h>
 
@@ -39,26 +38,21 @@ using namespace KABC;
 ResourceFileConfig::ResourceFileConfig( QWidget *parent )
     : ConfigWidget( parent )
 {
-  QGridLayout *mainLayout = new QGridLayout( this );
+  QFormLayout *mainLayout = new QFormLayout( this );
   mainLayout->setMargin( 0 );
-  mainLayout->setSpacing( KDialog::spacingHint() );
 
-  QLabel *label = new QLabel( i18n( "Format:" ), this );
   mFormatBox = new KComboBox( this );
 
-  mainLayout->addWidget( label, 0, 0 );
-  mainLayout->addWidget( mFormatBox, 0, 1 );
+  mainLayout->addRow( i18n( "Format:" ), mFormatBox );
 
-  label = new QLabel( i18n( "Location:" ), this );
   mFileNameEdit = new KUrlRequester( this );
   mFileNameEdit->setMode( KFile::File | KFile::LocalOnly );
 
+  mainLayout->addRow( i18n( "Location:" ), mFileNameEdit );
+
   connect( mFileNameEdit, SIGNAL( textChanged( const QString & ) ),
            SLOT( checkFilePermissions( const QString & ) ) );
 
-  mainLayout->addWidget( label, 1, 0 );
-  mainLayout->addWidget( mFileNameEdit, 1, 1 );
-
   FormatFactory *factory = FormatFactory::self();
   QStringList formats = factory->formats();
   QStringList::Iterator it;
@@ -84,7 +78,7 @@ void ResourceFileConfig::loadSettings( KRES::Resource *res )
   ResourceFile *resource = dynamic_cast<ResourceFile*>( res );
 
   if ( !resource ) {
-    kDebug(5700) << "ResourceFileConfig::loadSettings(): cast failed";
+    kDebug() << "cast failed";
     return;
   }
 
@@ -101,7 +95,7 @@ void ResourceFileConfig::saveSettings( KRES::Resource *res )
   ResourceFile *resource = dynamic_cast<ResourceFile*>( res );
 
   if ( !resource ) {
-    kDebug(5700) << "ResourceFileConfig::saveSettings(): cast failed";
+    kDebug() << "cast failed";
     return;
   }
 
