@@ -42,26 +42,17 @@ class Blogger1Private : public BlogPrivate
     Blogger1Private();
     virtual ~Blogger1Private();
 
-    virtual void slotFetchUserInfo( const QList<QVariant> &result,
-                                   const QVariant &id );
-    virtual void slotListBlogs( const QList<QVariant> &result,
-                                const QVariant &id );
-    virtual void slotListRecentPosts( const QList<QVariant> &result,
-                                         const QVariant &id );
-    virtual void slotFetchPost( const QList<QVariant> &result,
-                                   const QVariant &id );
-    virtual void slotCreatePost( const QList<QVariant> &result,
-                                    const QVariant &id );
-    virtual void slotModifyPost( const QList<QVariant> &result,
-                                    const QVariant &id );
-    virtual void slotRemovePost( const QList<QVariant> &result,
-                                    const QVariant &id );
-    virtual void slotError( int number, const QString &errorString,
-                            const QVariant &id );
+    virtual void slotFetchUserInfo( const QList<QVariant> &result, const QVariant &id );
+    virtual void slotListBlogs( const QList<QVariant> &result, const QVariant &id );
+    virtual void slotListRecentPosts( const QList<QVariant> &result, const QVariant &id );
+    virtual void slotFetchPost( const QList<QVariant> &result, const QVariant &id );
+    virtual void slotCreatePost( const QList<QVariant> &result, const QVariant &id );
+    virtual void slotModifyPost( const QList<QVariant> &result, const QVariant &id );
+    virtual void slotRemovePost( const QList<QVariant> &result, const QVariant &id );
+    virtual void slotError( int number, const QString &errorString, const QVariant &id );
 
-    Q_DECLARE_PUBLIC(KBlog::Blogger1)
+    Q_DECLARE_PUBLIC( KBlog::Blogger1 )
 
-//   private:
     enum FunctionToCall {
       GetRecentPosts,
       CreatePost,
@@ -72,7 +63,7 @@ class Blogger1Private : public BlogPrivate
     QList<QVariant> blogger1Args( const QString &id = QString() );
     virtual bool readPostFromMap( BlogPost *post,
                                      const QMap<QString, QVariant> &postInfo );
-    virtual bool readArgsFromPost( QList<QVariant> *args, const BlogPost& post );
+    virtual bool readArgsFromPost( QList<QVariant> *args, const BlogPost &post );
     virtual QString getCallFromFunction( FunctionToCall type );
 };
 
