@@ -26,6 +26,7 @@
 
 #include <syndication/loader.h>
 #include <syndication/item.h>
+#include <syndication/category.h>
 
 #include <kio/netaccess.h>
 #include <kio/http.h>
@@ -45,62 +46,60 @@ using namespace KBlog;
 GData::GData( const KUrl &server, QObject *parent )
   : Blog( server, *new GDataPrivate, parent )
 {
-  kDebug(5323);
+  kDebug();
   setUrl( server );
 }
 
 GData::~GData()
 {
-  kDebug(5323);
+  kDebug();
 }
 
 QString GData::interfaceName() const
 {
-  kDebug(5323);
+  kDebug();
   return QLatin1String( "Google Blogger Data" );
 }
 
 QString GData::fullName() const
 {
-  kDebug(5323);
+  kDebug();
   return d_func()->mFullName;
 }
 
 void GData::setFullName( const QString &fullName )
 {
-  kDebug(5323);
+  kDebug();
   Q_D( GData );
   d->mFullName = fullName;
 }
 
 QString GData::profileId() const
 {
-  kDebug(5323);
+  kDebug();
   return d_func()->mProfileId;
 }
 
 void GData::setProfileId( const QString &pid )
 {
-  kDebug(5323);
+  kDebug();
   Q_D( GData );
   d->mProfileId = pid;
 }
 
 void GData::fetchProfileId()
 {
-  kDebug(5323);
+  kDebug();
   QByteArray data;
-  KIO::Job *job = KIO::get( url(), KIO::NoReload, KIO::HideProgressInfo );
+  KIO::StoredTransferJob *job = KIO::storedGet( url(), KIO::NoReload, KIO::HideProgressInfo );
   KUrl blogUrl = url();
-  connect( job, SIGNAL(data(KIO::Job*,const QByteArray&)),
-           this, SLOT(slotFetchProfileIdData(KIO::Job*,const QByteArray&)) );
   connect( job, SIGNAL(result(KJob*)),
            this, SLOT(slotFetchProfileId(KJob*)) );
 }
 
 void GData::listBlogs()
 {
-  kDebug(5323);
+  kDebug();
   Syndication::Loader *loader = Syndication::Loader::create();
   connect( loader,
            SIGNAL(loadingComplete(Syndication::Loader*,Syndication::FeedPtr,Syndication::ErrorCode)),
@@ -113,7 +112,7 @@ void GData::listRecentPosts( const QStringList &labels, int number,
                              const KDateTime &upMinTime, const KDateTime &upMaxTime,
                              const KDateTime &pubMinTime, const KDateTime &pubMaxTime )
 {
-  kDebug(5323);
+  kDebug();
   Q_D( GData );
   QString urlString( "http://www.blogger.com/feeds/" + blogId() + "/posts/default" );
   if ( ! labels.empty() ) {
@@ -151,13 +150,13 @@ void GData::listRecentPosts( const QStringList &labels, int number,
 
 void GData::listRecentPosts( int number )
 {
-  kDebug(5323);
+  kDebug();
   listRecentPosts( QStringList(), number );
 }
 
 void GData::listComments( KBlog::BlogPost *post )
 {
-  kDebug(5323);
+  kDebug();
   Q_D( GData );
   Syndication::Loader *loader = Syndication::Loader::create();
   d->mListCommentsMap[ loader ] = post;
@@ -171,7 +170,7 @@ void GData::listComments( KBlog::BlogPost *post )
 
 void GData::listAllComments()
 {
-  kDebug(5323);
+  kDebug();
   Syndication::Loader *loader = Syndication::Loader::create();
   connect( loader,
            SIGNAL(loadingComplete(Syndication::Loader*,Syndication::FeedPtr,Syndication::ErrorCode)),
@@ -182,11 +181,11 @@ void GData::listAllComments()
 
 void GData::fetchPost( KBlog::BlogPost *post )
 {
-  kDebug(5323);
+  kDebug();
   Q_D( GData );
 
   if ( !post ) {
-    kError(5323) << "post is null pointer";
+    kError() << "post is null pointer";
     return;
   }
 
@@ -202,16 +201,16 @@ void GData::fetchPost( KBlog::BlogPost *post )
 
 void GData::modifyPost( KBlog::BlogPost *post )
 {
-  kDebug(5323);
+  kDebug();
   Q_D( GData );
 
   if ( !post ) {
-    kError(5323) << "post is null pointer";
+    kError() << "post is null pointer";
     return;
   }
 
   if ( !d->authenticate() ){
-    kError(5323) << "Authentication failed.";
+    kError() << "Authentication failed.";
     emit errorPost( Atom, i18n( "Authentication failed." ), post );
     return;
   }
@@ -230,6 +229,11 @@ void GData::modifyPost( KBlog::BlogPost *post )
   atomMarkup += "<div xmlns='http://www.w3.org/1999/xhtml'>";
   atomMarkup += post->content();
   atomMarkup += "</div></content>";
+  QList<QString>::ConstIterator it = post->tags().constBegin();
+  QList<QString>::ConstIterator end = post->tags().constEnd();
+  for( ; it != end; ++it ){
+    atomMarkup += "<category scheme='http://www.blogger.com/atom/ns#' term='" + ( *it ) + "' />";
+  }
   atomMarkup += "<author>";
   if ( !fullName().isEmpty() ) {
     atomMarkup += "<name>" + fullName() + "</name>";
@@ -241,9 +245,9 @@ void GData::modifyPost( KBlog::BlogPost *post )
   QDataStream stream( &postData, QIODevice::WriteOnly );
   stream.writeRawData( atomMarkup.toUtf8(), atomMarkup.toUtf8().length() );
 
-  KIO::TransferJob *job = KIO::http_post(
+  KIO::StoredTransferJob *job = KIO::storedHttpPost( postData,
       KUrl( "http://www.blogger.com/feeds/" + blogId() + "/posts/default/" + post->postId() ),
-      postData, KIO::HideProgressInfo );
+      KIO::HideProgressInfo );
 
   Q_ASSERT( job );
 
@@ -256,24 +260,22 @@ void GData::modifyPost( KBlog::BlogPost *post )
                     "Authorization: GoogleLogin auth=" + d->mAuthenticationString +
                     "\r\nX-HTTP-Method-Override: PUT" );
 
-  connect( job, SIGNAL(data(KIO::Job*,const QByteArray&)),
-           this, SLOT(slotModifyPostData(KIO::Job*,const QByteArray&)) );
   connect( job, SIGNAL(result(KJob*)),
            this, SLOT(slotModifyPost(KJob*)) );
 }
 
 void GData::createPost( KBlog::BlogPost *post )
 {
-  kDebug(5323);
+  kDebug();
   Q_D( GData );
 
   if ( !post ) {
-    kError(5323) << "post is null pointer";
+    kError() << "post is null pointer";
     return;
   }
 
   if ( !d->authenticate() ){
-    kError(5323) << "Authentication failed.";
+    kError() << "Authentication failed.";
     emit errorPost( Atom, i18n( "Authentication failed." ), post );
     return;
   }
@@ -288,6 +290,11 @@ void GData::createPost( KBlog::BlogPost *post )
   atomMarkup += "<div xmlns='http://www.w3.org/1999/xhtml'>";
   atomMarkup += post->content(); // FIXME check for Utf
   atomMarkup += "</div></content>";
+  QList<QString>::ConstIterator it = post->tags().constBegin();
+  QList<QString>::ConstIterator end = post->tags().constEnd();
+  for( ; it != end; ++it ){
+    atomMarkup += "<category scheme='http://www.blogger.com/atom/ns#' term='" + ( *it ) + "' />";
+  }
   atomMarkup += "<author>";
   if ( !fullName().isEmpty() ) {
     atomMarkup += "<name>" + fullName() + "</name>";
@@ -300,50 +307,44 @@ void GData::createPost( KBlog::BlogPost *post )
   QDataStream stream( &postData, QIODevice::WriteOnly );
   stream.writeRawData( atomMarkup.toUtf8(), atomMarkup.toUtf8().length() );
 
-  KIO::TransferJob *job = KIO::http_post(
+  KIO::StoredTransferJob *job = KIO::storedHttpPost( postData,
     KUrl( "http://www.blogger.com/feeds/" + blogId() + "/posts/default" ),
-    postData, KIO::HideProgressInfo );
+    KIO::HideProgressInfo );
 
+  Q_ASSERT ( job );
   d->mCreatePostMap[ job ] = post;
 
-  if ( !job ) {
-    kWarning() << "Unable to create KIO job for http://www.blogger.com/feeds/"
-               << blogId() << "/posts/default";
-  }
-
   job->addMetaData( "content-type", "Content-Type: application/atom+xml; charset=utf-8" );
   job->addMetaData( "ConnectTimeout", "50" );
   job->addMetaData( "UserAgent", userAgent() );
   job->addMetaData( "customHTTPHeader",
                     "Authorization: GoogleLogin auth=" + d->mAuthenticationString );
 
-  connect( job, SIGNAL(data(KIO::Job*,const QByteArray&)),
-           this, SLOT(slotCreatePostData(KIO::Job*,const QByteArray&)) );
   connect( job, SIGNAL(result(KJob*)),
            this, SLOT(slotCreatePost(KJob*)) );
 }
 
 void GData::removePost( KBlog::BlogPost *post )
 {
-  kDebug(5323);
+  kDebug();
   Q_D( GData );
 
   if ( !post ) {
-    kError(5323) << "post is null pointer";
+    kError() << "post is null pointer";
     return;
   }
 
   if ( !d->authenticate() ){
-    kError(5323) << "Authentication failed.";
+    kError() << "Authentication failed.";
     emit errorPost( Atom, i18n( "Authentication failed." ), post );
     return;
   }
 
   QByteArray postData;
 
-  KIO::TransferJob *job = KIO::http_post(
+  KIO::StoredTransferJob *job = KIO::storedHttpPost( postData,
     KUrl( "http://www.blogger.com/feeds/" + blogId() + "/posts/default/" + post->postId() ),
-    postData, KIO::HideProgressInfo );
+    KIO::HideProgressInfo );
 
   d->mRemovePostMap[ job ] = post;
 
@@ -358,29 +359,27 @@ void GData::removePost( KBlog::BlogPost *post )
                     "Authorization: GoogleLogin auth=" + d->mAuthenticationString +
                     "\r\nX-HTTP-Method-Override: DELETE" );
 
-  connect( job, SIGNAL(data(KIO::Job*,const QByteArray&)),
-           this, SLOT(slotRemovePostData(KIO::Job*,const QByteArray&)) );
   connect( job, SIGNAL(result(KJob*)),
            this, SLOT(slotRemovePost(KJob*)) );
 }
 
 void GData::createComment( KBlog::BlogPost *post, KBlog::BlogComment *comment )
 {
-  kDebug(5323);
+  kDebug();
 
   if ( !comment ) {
-    kError(5323) << "comment is null pointer";
+    kError() << "comment is null pointer";
     return;
   }
 
   if ( !post ) {
-    kError(5323) << "post is null pointer";
+    kError() << "post is null pointer";
     return;
   }
 
   Q_D( GData );
   if ( !d->authenticate() ){
-    kError(5323) << "Authentication failed.";
+    kError() << "Authentication failed.";
     emit errorComment( Atom, i18n( "Authentication failed." ), post, comment );
     return;
   }
@@ -397,9 +396,9 @@ void GData::createComment( KBlog::BlogPost *post, KBlog::BlogComment *comment )
   QDataStream stream( &postData, QIODevice::WriteOnly );
   stream.writeRawData( atomMarkup.toUtf8(), atomMarkup.toUtf8().length() );
 
-  KIO::TransferJob *job = KIO::http_post(
+  KIO::StoredTransferJob *job = KIO::storedHttpPost( postData,
     KUrl( "http://www.blogger.com/feeds/" + blogId() + "/" + post->postId() + "/comments/default" ),
-    postData, KIO::HideProgressInfo );
+    KIO::HideProgressInfo );
 
   d->mCreateCommentMap[ job ][post] = comment;
 
@@ -414,40 +413,37 @@ void GData::createComment( KBlog::BlogPost *post, KBlog::BlogComment *comment )
                     "Authorization: GoogleLogin auth=" + d->mAuthenticationString );
   job->addMetaData( "UserAgent", userAgent() );
 
-  connect( job, SIGNAL(data(KIO::Job*,const QByteArray&)),
-           this, SLOT(slotCreateCommentData(KIO::Job*,const QByteArray&)) );
   connect( job, SIGNAL(result(KJob*)),
            this, SLOT(slotCreateComment(KJob*)) );
 }
 
 void GData::removeComment( KBlog::BlogPost *post, KBlog::BlogComment *comment )
 {
-  kDebug(5323);
+  kDebug();
   Q_D( GData );
   kDebug();
 
   if ( !comment ) {
-    kError(5323) << "comment is null pointer";
+    kError() << "comment is null pointer";
     return;
   }
 
   if ( !post ) {
-    kError(5323) << "post is null pointer";
+    kError() << "post is null pointer";
     return;
   }
 
   if ( !d->authenticate() ){
-    kError(5323) << "Authentication failed.";
+    kError() << "Authentication failed.";
     emit errorComment( Atom, i18n( "Authentication failed." ), post, comment );
     return;
   }
 
   QByteArray postData;
 
-  KIO::TransferJob *job = KIO::http_post(
+  KIO::StoredTransferJob *job = KIO::storedHttpPost(postData,
     KUrl( "http://www.blogger.com/feeds/" + blogId() + "/" + post->postId() +
-          "/comments/default/" + comment->commentId() ),
-    postData, KIO::HideProgressInfo );
+          "/comments/default/" + comment->commentId() ), KIO::HideProgressInfo );
   d->mRemoveCommentMap[ job ][ post ] = comment;
 
   if ( !job ) {
@@ -462,25 +458,23 @@ void GData::removeComment( KBlog::BlogPost *post, KBlog::BlogComment *comment )
                     "Authorization: GoogleLogin auth=" +
                     d->mAuthenticationString + "\r\nX-HTTP-Method-Override: DELETE" );
 
-  connect( job, SIGNAL(data(KIO::Job*,const QByteArray&)),
-           this, SLOT(slotRemoveCommentData(KIO::Job*,const QByteArray&)) );
   connect( job, SIGNAL(result(KJob*)),
            this, SLOT(slotRemoveComment(KJob*)) );
 }
 
 GDataPrivate::GDataPrivate():mAuthenticationString(), mAuthenticationTime()
 {
-  kDebug(5323);
+  kDebug();
 }
 
 GDataPrivate::~GDataPrivate()
 {
-  kDebug(5323);
+  kDebug();
 }
 
 bool GDataPrivate::authenticate()
 {
-  kDebug(5323);
+  kDebug();
   Q_Q( GData );
   QByteArray data;
   KUrl authGateway( "https://www.google.com/accounts/ClientLogin" );
@@ -493,12 +487,9 @@ bool GDataPrivate::authenticate()
        mAuthenticationString.isEmpty() ) {
     KIO::Job *job = KIO::http_post( authGateway, QByteArray(), KIO::HideProgressInfo );
     if ( KIO::NetAccess::synchronousRun( job, (QWidget*)0, &data, &authGateway ) ) {
-      kDebug(5323) << "Fetched authentication result for"
-                   << authGateway.prettyUrl() << ".";
-      kDebug(5323) << "Authentication response:" << data;
       QRegExp rx( "Auth=(.+)" );
       if ( rx.indexIn( data ) != -1 ) {
-        kDebug(5323) << "RegExp got authentication string:" << rx.cap(1);
+        kDebug() << "RegExp got authentication string:" << rx.cap(1);
         mAuthenticationString = rx.cap(1);
         mAuthenticationTime = QDateTime::currentDateTime();
         return true;
@@ -509,54 +500,42 @@ bool GDataPrivate::authenticate()
   return true;
 }
 
-void GDataPrivate::slotFetchProfileIdData( KIO::Job *job, const QByteArray &data )
-{
-  kDebug(5323);
-  if( !job ){
-    kError(5323) << "job is a null pointer.";
-    return;
-  }
-  unsigned int oldSize = mFetchProfileIdBuffer[ job ].size();
-  mFetchProfileIdBuffer[ job ].resize( oldSize + data.size() );
-  memcpy( mFetchProfileIdBuffer[ job ].data() + oldSize, data.data(), data.size() );
-}
-
 void GDataPrivate::slotFetchProfileId( KJob *job )
 {
-  kDebug(5323);
+  kDebug();
   if( !job ){
-    kError(5323) << "job is a null pointer.";
+    kError() << "job is a null pointer.";
     return;
   }
   Q_Q( GData );
+  KIO::StoredTransferJob *stj = qobject_cast<KIO::StoredTransferJob*>(job);
+  const QString data = QString::fromUtf8( stj->data(), stj->data().size() );
   if ( !job->error() ) {
     QRegExp pid( "http://www.blogger.com/profile/(\\d+)" );
-    if ( pid.indexIn( mFetchProfileIdBuffer[ job ] ) != -1 ) {
+    if ( pid.indexIn( data ) != -1 ) {
       q->setProfileId( pid.cap(1) );
+      kDebug() << "QRegExp bid( 'http://www.blogger.com/profile/(\\d+)' matches" << pid.cap(1);
       emit q->fetchedProfileId( pid.cap(1) );
-      kDebug(5323) << "QRegExp bid( 'http://www.blogger.com/profile/(\\d+)' matches" << pid.cap(1);
     } else {
-      kError(5323) << "QRegExp bid( 'http://www.blogger.com/profile/(\\d+)' "
+      kError() << "QRegExp bid( 'http://www.blogger.com/profile/(\\d+)' "
                    << " could not regexp the Profile ID";
       emit q->error( GData::Other, i18n( "Could not regexp the Profile ID." ) );
       emit q->fetchedProfileId( QString() );
     }
   } else {
-    kError(5323) << "Could not fetch the homepage data.";
-    emit q->error( GData::Other, i18n( "Could not fetch the homepage data." ) );
+    kError() << "Job Error: "<<job->errorString();
+    emit q->error( GData::Other, job->errorString() );
     emit q->fetchedProfileId( QString() );
   }
-  mFetchProfileIdBuffer[ job ].resize( 0 );
-  mFetchProfileIdBuffer.remove( job );
 }
 
 void GDataPrivate::slotListBlogs( Syndication::Loader *loader,
                                   Syndication::FeedPtr feed,
                                   Syndication::ErrorCode status ) {
-  kDebug(5323);
+  kDebug();
   Q_Q( GData );
   if( !loader ) {
-    kError(5323) << "loader is a null pointer.";
+    kError() << "loader is a null pointer.";
     return;
   }
   if ( status != Syndication::Success ) {
@@ -567,35 +546,36 @@ void GDataPrivate::slotListBlogs( Syndication::Loader *loader,
   QList<QMap<QString,QString> > blogsList;
 
   QList<Syndication::ItemPtr> items = feed->items();
-  QList<Syndication::ItemPtr>::ConstIterator it = items.begin();
-  QList<Syndication::ItemPtr>::ConstIterator end = items.end();
+  QList<Syndication::ItemPtr>::ConstIterator it = items.constBegin();
+  QList<Syndication::ItemPtr>::ConstIterator end = items.constEnd();
   for ( ; it != end; ++it ) {
     QRegExp rx( "blog-(\\d+)" );
     QMap<QString,QString> blogInfo;
     if ( rx.indexIn( ( *it )->id() ) != -1 ) {
-      kDebug(5323) << "QRegExp rx( 'blog-(\\d+)' matches" << rx.cap(1);
+      kDebug() << "QRegExp rx( 'blog-(\\d+)' matches" << rx.cap(1);
       blogInfo["id"] = rx.cap(1);
       blogInfo["title"] = ( *it )->title();
+      blogInfo["url"] = ( *it )->link();
       blogInfo["summary"] = ( *it )->description(); //TODO fix/add more
       blogsList << blogInfo;
     } else {
+      kError() << "QRegExp rx( 'blog-(\\d+)' does not match anything in:"
+          << ( *it )->id();
       emit q->error( GData::Other, i18n( "Could not regexp the blog id path." ) );
-      kError(5323) << "QRegExp rx( 'blog-(\\d+)' does not match anything in:"
-                   << ( *it )->id();
     }
   }
+  kDebug() << "Emitting listedBlogs(); ";
   emit q->listedBlogs( blogsList );
-  kDebug(5323) << "Emitting listedBlogs(); ";
 }
 
 void GDataPrivate::slotListComments( Syndication::Loader *loader,
                                      Syndication::FeedPtr feed,
                                      Syndication::ErrorCode status )
 {
-  kDebug(5323);
+  kDebug();
   Q_Q( GData );
   if( !loader ) {
-    kError(5323) << "loader is a null pointer.";
+    kError() << "loader is a null pointer.";
     return;
   }
   BlogPost *post = mListCommentsMap[ loader ];
@@ -609,18 +589,18 @@ void GDataPrivate::slotListComments( Syndication::Loader *loader,
   QList<KBlog::BlogComment> commentList;
 
   QList<Syndication::ItemPtr> items = feed->items();
-  QList<Syndication::ItemPtr>::ConstIterator it = items.begin();
-  QList<Syndication::ItemPtr>::ConstIterator end = items.end();
+  QList<Syndication::ItemPtr>::ConstIterator it = items.constBegin();
+  QList<Syndication::ItemPtr>::ConstIterator end = items.constEnd();
   for ( ; it != end; ++it ) {
     BlogComment comment;
     QRegExp rx( "post-(\\d+)" );
     if ( rx.indexIn( ( *it )->id() ) == -1 ) {
-      kError(5323) << "QRegExp rx( 'post-(\\d+)' does not match" << rx.cap(1);
+      kError() << "QRegExp rx( 'post-(\\d+)' does not match" << rx.cap(1);
       emit q->error( GData::Other, i18n( "Could not regexp the comment id path." ) );
     } else {
       comment.setCommentId( rx.cap(1) );
     }
-    kDebug(5323) << "QRegExp rx( 'post-(\\d+)' matches" << rx.cap(1);
+    kDebug() << "QRegExp rx( 'post-(\\d+)' matches" << rx.cap(1);
     comment.setTitle( ( *it )->title() );
     comment.setContent( ( *it )->content() );
 //  FIXME: assuming UTC for now
@@ -632,7 +612,7 @@ void GDataPrivate::slotListComments( Syndication::Loader *loader,
                  KDateTime::Spec::UTC() ) );
     commentList.append( comment );
   }
-  kDebug(5323) << "Emitting listedComments()";
+  kDebug() << "Emitting listedComments()";
   emit q->listedComments( post, commentList );
 }
 
@@ -640,10 +620,10 @@ void GDataPrivate::slotListAllComments( Syndication::Loader *loader,
                                         Syndication::FeedPtr feed,
                                         Syndication::ErrorCode status )
 {
-  kDebug(5323);
+  kDebug();
   Q_Q( GData );
   if( !loader ) {
-    kError(5323) << "loader is a null pointer.";
+    kError() << "loader is a null pointer.";
     return;
   }
 
@@ -655,19 +635,19 @@ void GDataPrivate::slotListAllComments( Syndication::Loader *loader,
   QList<KBlog::BlogComment> commentList;
 
   QList<Syndication::ItemPtr> items = feed->items();
-  QList<Syndication::ItemPtr>::ConstIterator it = items.begin();
-  QList<Syndication::ItemPtr>::ConstIterator end = items.end();
+  QList<Syndication::ItemPtr>::ConstIterator it = items.constBegin();
+  QList<Syndication::ItemPtr>::ConstIterator end = items.constEnd();
   for ( ; it != end; ++it ) {
     BlogComment comment;
     QRegExp rx( "post-(\\d+)" );
     if ( rx.indexIn( ( *it )->id() ) == -1 ) {
-      kError(5323) << "QRegExp rx( 'post-(\\d+)' does not match"<< rx.cap(1);
+      kError() << "QRegExp rx( 'post-(\\d+)' does not match"<< rx.cap(1);
       emit q->error( GData::Other, i18n( "Could not regexp the comment id path." ) );
     } else {
       comment.setCommentId( rx.cap(1) );
     }
 
-    kDebug(5323) << "QRegExp rx( 'post-(\\d+)' matches" << rx.cap(1);
+    kDebug() << "QRegExp rx( 'post-(\\d+)' matches" << rx.cap(1);
     comment.setTitle( ( *it )->title() );
     comment.setContent( ( *it )->content() );
 //  FIXME: assuming UTC for now
@@ -679,17 +659,17 @@ void GDataPrivate::slotListAllComments( Syndication::Loader *loader,
                  KDateTime::Spec::UTC() ) );
     commentList.append( comment );
   }
-  kDebug(5323) << "Emitting listedAllComments()";
+  kDebug() << "Emitting listedAllComments()";
   emit q->listedAllComments( commentList );
 }
 
 void GDataPrivate::slotListRecentPosts( Syndication::Loader *loader,
                                         Syndication::FeedPtr feed,
                                         Syndication::ErrorCode status ) {
-  kDebug(5323);
+  kDebug();
   Q_Q( GData );
   if( !loader ) {
-    kError(5323) << "loader is a null pointer.";
+    kError() << "loader is a null pointer.";
     return;
   }
 
@@ -707,36 +687,47 @@ void GDataPrivate::slotListRecentPosts( Syndication::Loader *loader,
   QList<KBlog::BlogPost> postList;
 
   QList<Syndication::ItemPtr> items = feed->items();
-  QList<Syndication::ItemPtr>::ConstIterator it = items.begin();
-  QList<Syndication::ItemPtr>::ConstIterator end = items.end();
+  QList<Syndication::ItemPtr>::ConstIterator it = items.constBegin();
+  QList<Syndication::ItemPtr>::ConstIterator end = items.constEnd();
   for ( ; it != end; ++it ) {
     BlogPost post;
     QRegExp rx( "post-(\\d+)" );
     if ( rx.indexIn( ( *it )->id() ) == -1 ) {
-      kError(5323) << "QRegExp rx( 'post-(\\d+)' does not match"<< rx.cap(1);
+      kError() << "QRegExp rx( 'post-(\\d+)' does not match"<< rx.cap(1);
       emit q->error( GData::Other, i18n( "Could not regexp the post id path." ) );
     } else {
       post.setPostId( rx.cap(1) );
     }
 
-    kDebug(5323) << "QRegExp rx( 'post-(\\d+)' matches" << rx.cap(1);
+    kDebug() << "QRegExp rx( 'post-(\\d+)' matches" << rx.cap(1);
     post.setTitle( ( *it )->title() );
     post.setContent( ( *it )->content() );
     post.setLink( ( *it )->link() );
+    QStringList labels;
+    int catCount = ( *it )->categories().count();
+    QList< Syndication::CategoryPtr > cats = ( *it )->categories();
+    for(int i=0; i < catCount; ++i) {
+        if(cats[i].get()->label().isEmpty()){
+            labels.append(cats[i].get()->term());
+        } else {
+            labels.append(cats[i].get()->label());
+        }
+    }
+    post.setTags(labels);
 //  FIXME: assuming UTC for now
     post.setCreationDateTime(
       KDateTime( QDateTime::fromTime_t( ( *it )->datePublished() ),
-                 KDateTime::Spec::UTC() ) );
+                 KDateTime::Spec::UTC() ).toLocalZone() );
     post.setModificationDateTime(
       KDateTime( QDateTime::fromTime_t( ( *it )->dateUpdated() ),
-                 KDateTime::Spec::UTC() ) );
+                 KDateTime::Spec::UTC() ).toLocalZone() );
     post.setStatus( BlogPost::Fetched );
     postList.append( post );
     if ( number-- == 0 ) {
       break;
     }
   }
-  kDebug(5323) << "Emitting listedRecentPosts()";
+  kDebug() << "Emitting listedRecentPosts()";
   emit q->listedRecentPosts( postList );
 }
 
@@ -744,75 +735,65 @@ void GDataPrivate::slotFetchPost( Syndication::Loader *loader,
                                   Syndication::FeedPtr feed,
                                   Syndication::ErrorCode status )
 {
-  kDebug(5323);
+  kDebug();
   Q_Q( GData );
   if( !loader ) {
-    kError(5323) << "loader is a null pointer.";
+    kError() << "loader is a null pointer.";
     return;
   }
 
   bool success = false;
 
-  BlogPost *post = mFetchPostMap[ loader ];
+  BlogPost *post = mFetchPostMap.take( loader );
+  kError() << "Post" << post;
+  post->postId();
 
   if ( status != Syndication::Success ) {
     emit q->errorPost( GData::Atom, i18n( "Could not get posts." ), post );
     return;
   }
+
+  QString postId = post->postId();
   QList<Syndication::ItemPtr> items = feed->items();
-  QList<Syndication::ItemPtr>::ConstIterator it = items.begin();
-  QList<Syndication::ItemPtr>::ConstIterator end = items.end();
+  QList<Syndication::ItemPtr>::ConstIterator it = items.constBegin();
+  QList<Syndication::ItemPtr>::ConstIterator end = items.constEnd();
   for ( ; it != end; ++it ) {
     QRegExp rx( "post-(\\d+)" );
-    if ( rx.indexIn( ( *it )->id() ) != -1 && rx.cap(1) == post->postId() ){
-      kDebug(5323) << "QRegExp rx( 'post-(\\d+)' matches" << rx.cap(1);
+    if ( rx.indexIn( ( *it )->id() ) != -1 && rx.cap(1) == postId ){
+      kDebug() << "QRegExp rx( 'post-(\\d+)' matches" << rx.cap(1);
       post->setPostId( rx.cap(1) );
       post->setTitle( ( *it )->title() );
       post->setContent( ( *it )->content() );
       post->setStatus( BlogPost::Fetched );
       post->setLink( ( *it )->link() );
-//    FIXME: assuming UTC for now
       post->setCreationDateTime(
         KDateTime( QDateTime::fromTime_t( ( *it )->datePublished() ),
-                   KDateTime::Spec::UTC() ) );
+                   KDateTime::Spec::UTC() ).toLocalZone() );
       post->setModificationDateTime(
         KDateTime( QDateTime::fromTime_t( ( *it )->dateUpdated() ),
-                   KDateTime::Spec::UTC() ) );
-      emit q->fetchedPost( post );
+                   KDateTime::Spec::UTC() ).toLocalZone() );
+      kDebug() << "Emitting fetchedPost( postId=" << postId << ");";
       success = true;
-      kDebug(5323) << "Emitting fetchedPost( postId=" << post->postId() << ");";
+      emit q->fetchedPost( post );
+      break;
     }
   }
   if ( !success ) {
+    kError() << "QRegExp rx( 'post-(\\d+)' does not match"
+        << mFetchPostMap[ loader ]->postId() << ".";
     emit q->errorPost( GData::Other, i18n( "Could not regexp the blog id path." ), post );
-    kError(5323) << "QRegExp rx( 'post-(\\d+)' does not match"
-                 << mFetchPostMap[ loader ]->postId() << ".";
   }
-  mFetchPostMap.remove( loader );
-}
-
-void GDataPrivate::slotCreatePostData( KIO::Job *job, const QByteArray &data )
-{
-  kDebug(5323);
-  if( !job ) {
-    kError(5323) << "job is a null pointer.";
-    return;
-  }
-  unsigned int oldSize = mCreatePostBuffer[ job ].size();
-  mCreatePostBuffer[ job ].resize( oldSize + data.size() );
-  memcpy( mCreatePostBuffer[ job ].data() + oldSize, data.data(), data.size() );
 }
 
 void GDataPrivate::slotCreatePost( KJob *job )
 {
-  kDebug(5323);
+  kDebug();
   if( !job ) {
-    kError(5323) << "job is a null pointer.";
+    kError() << "job is a null pointer.";
     return;
   }
-  const QString data = QString::fromUtf8( mCreatePostBuffer[ job ].data(),
-                                          mCreatePostBuffer[ job ].size() );
-  mCreatePostBuffer[ job ].resize( 0 );
+  KIO::StoredTransferJob *stj = qobject_cast<KIO::StoredTransferJob*>(job);
+  const QString data = QString::fromUtf8( stj->data(), stj->data().size() );
 
   Q_Q( GData );
 
@@ -820,104 +801,91 @@ void GDataPrivate::slotCreatePost( KJob *job )
   mCreatePostMap.remove( job );
 
   if ( job->error() != 0 ) {
-    kError(5323) << "slotCreatePost error:" << job->errorString();
+    kError() << "slotCreatePost error:" << job->errorString();
     emit q->errorPost( GData::Atom, job->errorString(), post );
     return;
   }
 
   QRegExp rxId( "post-(\\d+)" ); //FIXME check and do better handling, esp the creation date time
   if ( rxId.indexIn( data ) == -1 ) {
-    kError(5323) << "Could not regexp the id out of the result:" << data;
+    kError() << "Could not regexp the id out of the result:" << data;
     emit q->errorPost( GData::Atom,
                        i18n( "Could not regexp the id out of the result." ), post );
     return;
   }
-  kDebug(5323) << "QRegExp rx( 'post-(\\d+)' ) matches" << rxId.cap(1);
+  kDebug() << "QRegExp rx( 'post-(\\d+)' ) matches" << rxId.cap(1);
 
   QRegExp rxPub( "<published>(.+)</published>" );
   if ( rxPub.indexIn( data ) == -1 ) {
-    kError(5323) << "Could not regexp the published time out of the result:" << data;
+    kError() << "Could not regexp the published time out of the result:" << data;
     emit q->errorPost( GData::Atom,
                        i18n( "Could not regexp the published time out of the result." ), post );
     return;
   }
-  kDebug(5323) << "QRegExp rx( '<published>(.+)</published>' ) matches" << rxPub.cap(1);
+  kDebug() << "QRegExp rx( '<published>(.+)</published>' ) matches" << rxPub.cap(1);
 
   QRegExp rxUp( "<updated>(.+)</updated>" );
   if ( rxUp.indexIn( data ) == -1 ) {
-    kError(5323) << "Could not regexp the update time out of the result:" << data;
+    kError() << "Could not regexp the update time out of the result:" << data;
     emit q->errorPost( GData::Atom,
                        i18n( "Could not regexp the update time out of the result." ), post );
     return;
   }
-  kDebug(5323) << "QRegExp rx( '<updated>(.+)</updated>' ) matches" << rxUp.cap(1);
+  kDebug() << "QRegExp rx( '<updated>(.+)</updated>' ) matches" << rxUp.cap(1);
 
   post->setPostId( rxId.cap(1) );
-  post->setCreationDateTime( KDateTime().fromString( rxPub.cap(1) ) );
+  post->setCreationDateTime( KDateTime().fromString( rxPub.cap(1) ).toLocalZone() );
   post->setModificationDateTime( KDateTime().fromString( rxUp.cap(1) ) );
   post->setStatus( BlogPost::Created );
+  kDebug() << "Emitting createdPost()";
   emit q->createdPost( post );
-  kDebug(5323) << "Emitting createdPost()";
-}
-
-void GDataPrivate::slotModifyPostData( KIO::Job *job, const QByteArray &data )
-{
-  kDebug(5323);
-  if( !job ) {
-    kError(5323) << "job is a null pointer.";
-    return;
-  }
-  unsigned int oldSize = mModifyPostBuffer[ job ].size();
-  mModifyPostBuffer[ job ].resize( oldSize + data.size() );
-  memcpy( mModifyPostBuffer[ job ].data() + oldSize, data.data(), data.size() );
 }
 
 void GDataPrivate::slotModifyPost( KJob *job )
 {
-  kDebug(5323);
+  kDebug();
   if( !job ) {
-    kError(5323) << "job is a null pointer.";
+    kError() << "job is a null pointer.";
     return;
   }
-  const QString data = QString::fromUtf8( mModifyPostBuffer[ job ].data(),
-                                          mModifyPostBuffer[ job ].size() );
-  mModifyPostBuffer[ job ].resize( 0 );
+  KIO::StoredTransferJob *stj = qobject_cast<KIO::StoredTransferJob*>(job);
+  const QString data = QString::fromUtf8( stj->data(), stj->data().size() );
 
   KBlog::BlogPost *post = mModifyPostMap[ job ];
   mModifyPostMap.remove( job );
   Q_Q( GData );
   if ( job->error() != 0 ) {
-    kError(5323) << "slotModifyPost error:" << job->errorString();
+    kError() << "slotModifyPost error:" << job->errorString();
     emit q->errorPost( GData::Atom, job->errorString(), post );
     return;
   }
 
   QRegExp rxId( "post-(\\d+)" ); //FIXME check and do better handling, esp creation date time
   if ( rxId.indexIn( data ) == -1 ) {
-    kError(5323) << "Could not regexp the id out of the result:" << data;
+    kError() << "Could not regexp the id out of the result:" << data;
     emit q->errorPost( GData::Atom,
                        i18n( "Could not regexp the id out of the result." ), post );
     return;
   }
-  kDebug(5323) << "QRegExp rx( 'post-(\\d+)' ) matches" << rxId.cap(1);
+  kDebug() << "QRegExp rx( 'post-(\\d+)' ) matches" << rxId.cap(1);
 
   QRegExp rxPub( "<published>(.+)</published>" );
   if ( rxPub.indexIn( data ) == -1 ) {
-    kError(5323) << "Could not regexp the published time out of the result:" << data;
+    kError() << "Could not regexp the published time out of the result:" << data;
     emit q->errorPost( GData::Atom,
                        i18n( "Could not regexp the published time out of the result." ), post );
     return;
   }
-  kDebug(5323) << "QRegExp rx( '<published>(.+)</published>' ) matches" << rxPub.cap(1);
+  kDebug() << "QRegExp rx( '<published>(.+)</published>' ) matches" << rxPub.cap(1);
 
   QRegExp rxUp( "<updated>(.+)</updated>" );
   if ( rxUp.indexIn( data ) == -1 ) {
-    kError(5323) << "Could not regexp the update time out of the result:" << data;
+    kError() << "Could not regexp the update time out of the result:" << data;
     emit q->errorPost( GData::Atom,
                        i18n( "Could not regexp the update time out of the result." ), post );
     return;
   }
-  kDebug(5323) << "QRegExp rx( '<updated>(.+)</updated>' ) matches" << rxUp.cap(1);
+  kDebug() << "QRegExp rx( '<updated>(.+)</updated>' ) matches" << rxUp.cap(1);
   post->setPostId( rxId.cap(1) );
   post->setCreationDateTime( KDateTime().fromString( rxPub.cap(1) ) );
   post->setModificationDateTime( KDateTime().fromString( rxUp.cap(1) ) );
@@ -925,66 +893,40 @@ void GDataPrivate::slotModifyPost( KJob *job )
   emit q->modifiedPost( post );
 }
 
-void GDataPrivate::slotRemovePostData( KIO::Job *job, const QByteArray &data )
-{
-  kDebug(5323);
-  if( !job ) {
-    kError(5323) << "job is a null pointer.";
-    return;
-  }
-  unsigned int oldSize = mRemovePostBuffer[ job ].size();
-  mRemovePostBuffer[ job ].resize( oldSize + data.size() );
-  memcpy( mRemovePostBuffer[ job ].data() + oldSize, data.data(), data.size() );
-}
-
 void GDataPrivate::slotRemovePost( KJob *job )
 {
-  kDebug(5323);
+  kDebug();
   if( !job ) {
-    kError(5323) << "job is a null pointer.";
+    kError() << "job is a null pointer.";
     return;
   }
-  const QString data = QString::fromUtf8( mRemovePostBuffer[ job ].data(),
-                                          mRemovePostBuffer[ job ].size() );
-  mRemovePostBuffer[ job ].resize( 0 );
+  KIO::StoredTransferJob *stj = qobject_cast<KIO::StoredTransferJob*>(job);
+  const QString data = QString::fromUtf8( stj->data(), stj->data().size() );
 
   KBlog::BlogPost *post = mRemovePostMap[ job ];
   mRemovePostMap.remove( job );
   Q_Q( GData );
   if ( job->error() != 0 ) {
-    kError(5323) << "slotRemovePost error:" << job->errorString();
+    kError() << "slotRemovePost error:" << job->errorString();
     emit q->errorPost( GData::Atom, job->errorString(), post );
     return;
   }
 
   post->setStatus( BlogPost::Removed );
+  kDebug() << "Emitting removedPost()";
   emit q->removedPost( post );
-  kDebug(5323) << "Emitting removedPost()";
-}
-
-void GDataPrivate::slotCreateCommentData( KIO::Job *job, const QByteArray &data )
-{
-  kDebug(5323);
-  if( !job ) {
-    kError(5323) << "job is a null pointer.";
-    return;
-  }
-  unsigned int oldSize = mCreateCommentBuffer[ job ].size();
-  mCreateCommentBuffer[ job ].resize( oldSize + data.size() );
-  memcpy( mCreateCommentBuffer[ job ].data() + oldSize, data.data(), data.size() );
 }
 
 void GDataPrivate::slotCreateComment( KJob *job )
 {
-  kDebug(5323);
+  kDebug();
   if( !job ) {
-    kError(5323) << "job is a null pointer.";
+    kError() << "job is a null pointer.";
     return;
   }
-  const QString data = QString::fromUtf8( mCreateCommentBuffer[ job ].data(),
-                                          mCreateCommentBuffer[ job ].size() );
-  mCreateCommentBuffer[ job ].resize( 0 );
-  kDebug(5323) << "Dump data: " << data;
+  KIO::StoredTransferJob *stj = qobject_cast<KIO::StoredTransferJob*>(job);
+  const QString data = QString::fromUtf8( stj->data(), stj->data().size() );
+  kDebug() << "Dump data: " << data;
 
   Q_Q( GData );
 
@@ -993,7 +935,7 @@ void GDataPrivate::slotCreateComment( KJob *job )
   mCreateCommentMap.remove( job );
 
   if ( job->error() != 0 ) {
-    kError(5323) << "slotCreateComment error:" << job->errorString();
+    kError() << "slotCreateComment error:" << job->errorString();
     emit q->errorComment( GData::Atom, job->errorString(), post, comment );
     return;
   }
@@ -1001,60 +943,47 @@ void GDataPrivate::slotCreateComment( KJob *job )
 // TODO check for result and fit appropriately
   QRegExp rxId( "post-(\\d+)" );
   if ( rxId.indexIn( data ) == -1 ) {
-    kError(5323) << "Could not regexp the id out of the result:" << data;
+    kError() << "Could not regexp the id out of the result:" << data;
     emit q->errorPost( GData::Atom,
                        i18n( "Could not regexp the id out of the result." ), post );
     return;
   }
-  kDebug(5323) << "QRegExp rx( 'post-(\\d+)' ) matches" << rxId.cap(1);
+  kDebug() << "QRegExp rx( 'post-(\\d+)' ) matches" << rxId.cap(1);
 
   QRegExp rxPub( "<published>(.+)</published>" );
   if ( rxPub.indexIn( data ) == -1 ) {
-    kError(5323) << "Could not regexp the published time out of the result:" << data;
+    kError() << "Could not regexp the published time out of the result:" << data;
     emit q->errorPost( GData::Atom,
                        i18n( "Could not regexp the published time out of the result." ), post );
     return;
   }
-  kDebug(5323) << "QRegExp rx( '<published>(.+)</published>' ) matches" << rxPub.cap(1);
+  kDebug() << "QRegExp rx( '<published>(.+)</published>' ) matches" << rxPub.cap(1);
 
   QRegExp rxUp( "<updated>(.+)</updated>" );
   if ( rxUp.indexIn( data ) == -1 ) {
-    kError(5323) << "Could not regexp the update time out of the result:" << data;
+    kError() << "Could not regexp the update time out of the result:" << data;
     emit q->errorPost( GData::Atom,
                        i18n( "Could not regexp the update time out of the result." ), post );
     return;
   }
-  kDebug(5323) << "QRegExp rx( '<updated>(.+)</updated>' ) matches" << rxUp.cap(1);
+  kDebug() << "QRegExp rx( '<updated>(.+)</updated>' ) matches" << rxUp.cap(1);
   comment->setCommentId( rxId.cap(1) );
   comment->setCreationDateTime( KDateTime().fromString( rxPub.cap(1) ) );
   comment->setModificationDateTime( KDateTime().fromString( rxUp.cap(1) ) );
   comment->setStatus( BlogComment::Created );
+  kDebug() << "Emitting createdComment()";
   emit q->createdComment( post, comment );
-  kDebug(5323) << "Emitting createdComment()";
-}
-
-void GDataPrivate::slotRemoveCommentData( KIO::Job *job, const QByteArray &data )
-{
-  kDebug(5323);
-  if( !job ) {
-    kError(5323) << "job is a null pointer.";
-    return;
-  }
-  unsigned int oldSize = mRemoveCommentBuffer[ job ].size();
-  mRemoveCommentBuffer[ job ].resize( oldSize + data.size() );
-  memcpy( mRemoveCommentBuffer[ job ].data() + oldSize, data.data(), data.size() );
 }
 
 void GDataPrivate::slotRemoveComment( KJob *job )
 {
-  kDebug(5323);
+  kDebug();
   if( !job ) {
-    kError(5323) << "job is a null pointer.";
+    kError() << "job is a null pointer.";
     return;
   }
-  const QString data = QString::fromUtf8( mRemoveCommentBuffer[ job ].data(),
-                                          mRemoveCommentBuffer[ job ].size() );
-  mRemoveCommentBuffer[ job ].resize( 0 );
+  KIO::StoredTransferJob *stj = qobject_cast<KIO::StoredTransferJob*>(job);
+  const QString data = QString::fromUtf8( stj->data(), stj->data().size() );
 
   Q_Q( GData );
 
@@ -1063,14 +992,14 @@ void GDataPrivate::slotRemoveComment( KJob *job )
   mRemoveCommentMap.remove( job );
 
   if ( job->error() != 0 ) {
-    kError(5323) << "slotRemoveComment error:" << job->errorString();
+    kError() << "slotRemoveComment error:" << job->errorString();
     emit q->errorComment( GData::Atom, job->errorString(), post, comment );
     return;
   }
 
   comment->setStatus( BlogComment::Created );
+  kDebug() << "Emitting removedComment()";
   emit q->removedComment( post, comment );
-  kDebug(5323) << "Emitting removedComment()";
 }
 
 #include "gdata.moc"
