@@ -75,6 +75,9 @@ class KCal::Calendar::Private
     ~Private()
     {
       delete mTimeZones;
+      if ( mFilter != mDefaultFilter ) {
+        delete mFilter;
+      }
       delete mDefaultFilter;
     }
     KDateTime::Spec timeZoneIdSpec( const QString &timeZoneId, bool view );
@@ -790,6 +793,16 @@ Journal::List Calendar::journals( const QDate &date )
   return el;
 }
 
+void Calendar::beginBatchAdding()
+{
+  emit batchAddingBegins();
+}
+
+void Calendar::endBatchAdding()
+{
+  emit batchAddingEnds();
+}
+
 // When this is called, the to-dos have already been added to the calendar.
 // This method is only about linking related to-dos.
 void Calendar::setupRelations( Incidence *forincidence )
@@ -895,14 +908,14 @@ void Calendar::removeRelations( Incidence *incidence )
     }
   }
 
-  // Make sure the deleted incidence doesn't relate to a non-deleted incidence, since
-  // that would cause trouble in CalendarLocal::close(), as the deleted incidences
-  // are destroyed after the non-deleted incidences. The destructor of the deleted
-  // incidences would then try to access the already destroyed non-deleted incidence,
-  // which would segfault.
+  // Make sure the deleted incidence doesn't relate to a non-deleted incidence,
+  // since that would cause trouble in CalendarLocal::close(), as the deleted
+  // incidences are destroyed after the non-deleted incidences. The destructor
+  // of the deleted incidences would then try to access the already destroyed
+  // non-deleted incidence, which would segfault.
   //
-  // So in short: Make sure dead incidences don't point to alive incidences via the
-  // relation.
+  // So in short: Make sure dead incidences don't point to alive incidences
+  // via the relation.
   //
   // This crash is tested in CalendarLocalTest::testRelationsCrash().
   incidence->setRelatedTo( 0 );
