@@ -74,6 +74,11 @@ CustomProperties::CustomProperties( const CustomProperties &cp )
 
 CustomProperties &CustomProperties::operator=( const CustomProperties &other )
 {
+  // check for self assignment
+  if ( &other == this ) {
+    return *this;
+  }
+
   *d = *other.d;
   return *this;
 }
@@ -123,20 +128,14 @@ void CustomProperties::setNonKDECustomProperty( const QByteArray &name, const QS
 
 void CustomProperties::removeNonKDECustomProperty( const QByteArray &name )
 {
-  QMap<QByteArray, QString>::Iterator it = d->mProperties.find( name );
-  if ( it != d->mProperties.end() ) {
-    d->mProperties.erase( it );
+  if ( d->mProperties.remove( name ) ) {
     customPropertyUpdated();
   }
 }
 
 QString CustomProperties::nonKDECustomProperty( const QByteArray &name ) const
 {
-  QMap<QByteArray, QString>::ConstIterator it = d->mProperties.find( name );
-  if ( it == d->mProperties.end() ) {
-    return QString();
-  }
-  return it.value();
+  return d->mProperties.value( name );
 }
 
 void CustomProperties::setCustomProperties( const QMap<QByteArray, QString> &properties )
@@ -172,9 +171,9 @@ bool checkName( const QByteArray &name )
   }
   for ( int i = 2;  i < len;  ++i ) {
     char ch = n[i];
-    if ( ch >= 'A' && ch <= 'Z' ||
-         ch >= 'a' && ch <= 'z' ||
-         ch >= '0' && ch <= '9' ||
+    if ( ( ch >= 'A' && ch <= 'Z' ) ||
+         ( ch >= 'a' && ch <= 'z' ) ||
+         ( ch >= '0' && ch <= '9' ) ||
          ch == '-' ) {
       continue;
     }
