@@ -101,10 +101,21 @@ class KCAL_EXPORT ResourceCalendar : public KRES::Resource
       to do the actual saving.
 
       @param incidence if given as 0, doSave() is called to save all incidences,
-             else doSave(incidence) is called to save only the given one
+             else doSave(incidence) is called to save only the given one.
     */
     bool save( Incidence *incidence = 0 );
 
+    /**
+      Save resource data and deliver error message.
+
+      This function calls save() and delivers the latest error msg.
+
+      @param err reference QString to hand over the error msg to the caller.
+      @param incidence if given as 0, doSave() is called to save all incidences,
+             else doSave(incidence) is called to save only the given one.
+    */
+    bool save( QString &err, Incidence *incidence = 0 );
+
     /**
       Return true if a save operation is still in progress, otherwise return
       false.
@@ -184,13 +195,13 @@ class KCAL_EXPORT ResourceCalendar : public KRES::Resource
       date specified. Useful for dayView, etc. etc.
 
       @param date date for which to get the events
-      @param timespec the time specification of the date
+      @param timeSpec the time specification of the date
       @param sortField field used as the sort key for the result list
       @param sortDirection direction of sorting according to @p sortField
     */
     virtual Event::List rawEventsForDate(
       const QDate &date,
-      const KDateTime::Spec &timespec = KDateTime::Spec(),
+      const KDateTime::Spec &timeSpec = KDateTime::Spec(),
       EventSortField sortField = EventSortUnsorted,
       SortDirection sortDirection = SortDirectionAscending ) = 0;
 
@@ -211,7 +222,7 @@ class KCAL_EXPORT ResourceCalendar : public KRES::Resource
     */
     virtual Event::List rawEvents(
       const QDate &start, const QDate &end,
-      const KDateTime::Spec &timespec = KDateTime::Spec(),
+      const KDateTime::Spec &timeSpec = KDateTime::Spec(),
       bool inclusive = false ) = 0;
 
     /**
