@@ -1,6 +1,6 @@
 /*
   This file is part of the kcal library.
-  Copyright (C) 2006 Allen Winter <winter@kde.org>
+  Copyright (C) 2006,2008 Allen Winter <winter@kde.org>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
@@ -17,10 +17,9 @@
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 */
-#include <qtest_kde.h>
-
 #include "testtodo.h"
 #include "testtodo.moc"
+#include <qtest_kde.h>
 
 QTEST_KDEMAIN( TodoTest, NoGUI )
 
@@ -37,6 +36,7 @@ void TodoTest::testValidity()
   todo->setDescription( "This is a description of the first to-do" );
   todo->setLocation( "the place" );
   todo->setPercentComplete( 5 );
+  //KDE5: QVERIFY( todo->typeStr() == i18n( "to-do" ) );
   QVERIFY( todo->summary() == "To-do1 Summary" );
   QVERIFY( todo->location() == "the place" );
   QVERIFY( todo->percentComplete() == 5 );
@@ -85,3 +85,58 @@ void TodoTest::testClone()
   QVERIFY( todo1.location( ) == todo2->location() );
   QVERIFY( todo1.isCompleted() == todo2->isCompleted() );
 }
+
+void TodoTest::testAssign()
+{
+  QDate dt = QDate::currentDate();
+  Todo todo1;
+  todo1.setDtStart( KDateTime( dt ) );
+  todo1.setDtDue( KDateTime( dt ).addDays( 1 ) );
+  todo1.setSummary( "Todo1 Summary" );
+  todo1.setDescription( "This is a description of the first todo" );
+  todo1.setLocation( "the place" );
+
+  Todo todo2 = todo1;
+  QVERIFY( todo1 == todo2 );
+}
+
+void TodoTest::testSetCompleted() {
+
+  Todo todo1, todo2;
+  todo1.setSummary( "Todo Summary" );
+  todo2.setSummary( "Todo Summary" );
+  KDateTime today = KDateTime::currentUtcDateTime();
+
+  // due yesterday
+  KDateTime originalDueDate = today.addDays( -1 );
+
+  todo1.setDtDue( originalDueDate );
+  todo1.recurrence()->setDaily( 1 );
+  todo1.setCompleted( today );
+
+  todo2.setCompleted( true );
+
+  QVERIFY( originalDueDate != todo1.dtDue() );
+  QVERIFY( !todo1.isCompleted() );
+  QVERIFY( todo2.isCompleted() );
+}
+
+void TodoTest::testStatus() {
+  KDateTime today = KDateTime::currentUtcDateTime();
+  KDateTime yesterday = today.addDays( -1 );
+
+  Todo todo1;
+  todo1.setDtStart( yesterday );
+  todo1.setDtDue( today );
+  todo1.setPercentComplete( 50 );
+  QVERIFY( todo1.isInProgress( false ) );
+  QVERIFY( !todo1.isNotStarted( false ) );
+  QVERIFY( !todo1.isOverdue() );
+  todo1.setPercentComplete( 100 );
+  QVERIFY( todo1.isCompleted() );
+
+  Todo todo2 = todo1;
+  todo2.setPercentComplete( 33 );
+  todo2.setHasDueDate( false );
+  QVERIFY( todo2.isOpenEnded() );
+}
