@@ -14,9 +14,9 @@
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *   GNU General Public License for more details.
  *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *   You should have received a copy of the GNU General Public License along
+ *   with this program; if not, write to the Free Software Foundation, Inc.,
+ *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  *   Send comments and bug fixes to jcorey@fruity.ath.cx
  *
@@ -67,12 +67,11 @@ imap://server/folder/
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <errno.h>
 
-#ifdef HAVE_LIBSASL2
 extern "C" {
 #include <sasl/sasl.h>
 }
-#endif
 
 #include <qbuffer.h>
 #include <qdatetime.h>
@@ -87,13 +86,15 @@ extern "C" {
 #include <klocale.h>
 #include <kmimetype.h>
 #include <kcodecs.h>
+#include <kde_file.h>
 
+#include "common.h"
 #include "kdemacros.h"
 
 #define IMAP_PROTOCOL "imap"
 #define IMAP_SSL_PROTOCOL "imaps"
-#define IMAP_PORT 143
-#define IMAPS_PORT 993
+const int ImapPort = 143;
+const int ImapsPort = 993;
 
 using namespace KIO;
 
@@ -106,21 +107,17 @@ extern "C"
 int
 kdemain (int argc, char **argv)
 {
+  KComponentData instance ("kio_imap4");
   kDebug(7116) <<"IMAP4::kdemain";
 
-  KComponentData instance ("kio_imap4");
   if (argc != 4)
   {
     fprintf(stderr, "Usage: kio_imap4 protocol domain-socket1 domain-socket2\n");
     ::exit (-1);
   }
 
-#ifdef HAVE_LIBSASL2
-  if ( sasl_client_init( NULL ) != SASL_OK ) {
-    fprintf(stderr, "SASL library initialization failed!\n");
-    ::exit (-1);
-  }
-#endif
+  if (!initSASL())
+    ::exit(-1);
 
   //set debug handler
 
@@ -134,9 +131,7 @@ kdemain (int argc, char **argv)
   slave->dispatchLoop ();
   delete slave;
 
-#ifdef HAVE_LIBSASL2
   sasl_done();
-#endif
 
   return 0;
 }
@@ -144,6 +139,10 @@ kdemain (int argc, char **argv)
 void
 sigchld_handler (int signo)
 {
+  // A signal handler that calls for example waitpid has to save errno
+  // before and restore it afterwards.
+  // (cf. https://www.securecoding.cert.org/confluence/display/cplusplus/ERR32-CPP.+Do+not+rely+on+indeterminate+values+of+errno)
+  const int save_errno = errno;
   int pid, status;
 
   while (signo == SIGCHLD)
@@ -154,10 +153,12 @@ sigchld_handler (int signo)
       // Reinstall signal handler, since Linux resets to default after
       // the signal occurred ( BSD handles it different, but it should do
       // no harm ).
-      signal (SIGCHLD, sigchld_handler);
-      return;
+      KDE_signal (SIGCHLD, sigchld_handler);
+      break;
     }
   }
+
+  errno = save_errno;
 }
 
 IMAP4Protocol::IMAP4Protocol (const QByteArray & pool, const QByteArray & app, bool isSSL)
@@ -198,7 +199,7 @@ IMAP4Protocol::get (const KUrl & _url)
 
   if (aSequence == "0:0" && getState() == ISTATE_SELECT)
   {
-    imapCommand *cmd = doCommand (imapCommand::clientNoop());
+    CommandPtr cmd = doCommand (imapCommand::clientNoop());
     completeQueue.removeAll(cmd);
   }
 
@@ -208,7 +209,7 @@ IMAP4Protocol::get (const KUrl & _url)
   }
 
   mProcessedSize = 0;
-  imapCommand *cmd = NULL;
+  CommandPtr cmd;
   if (!assureBox (aBox, true)) return;
 
 #ifdef USE_VALIDITY
@@ -311,7 +312,7 @@ IMAP4Protocol::get (const KUrl & _url)
       {
         // get the MIME header and fill getLastHandled()
         QString mySection = aSection;
-        mySection.replace("]", ".MIME]");
+        mySection.replace(']', ".MIME]");
         cmd = sendCommand (imapCommand::clientFetch (aSequence, mySection));
         do
         {
@@ -420,7 +421,7 @@ IMAP4Protocol::listDir (const KUrl & _url)
   if (myType == ITYPE_DIR || myType == ITYPE_DIR_AND_BOX)
   {
     QString listStr = myBox;
-    imapCommand *cmd;
+    CommandPtr cmd;
 
     if (!listStr.isEmpty () && !listStr.endsWith(myDelimiter) &&
         mySection != "FOLDERONLY")
@@ -505,7 +506,7 @@ IMAP4Protocol::listDir (const KUrl & _url)
       query = query.right (query.length () - 1);
       if (!query.isEmpty())
       {
-        imapCommand *cmd = NULL;
+        CommandPtr cmd;
 
         if (!assureBox (myBox, true)) return;
 
@@ -528,7 +529,7 @@ IMAP4Protocol::listDir (const KUrl & _url)
           UDSEntry entry;
           imapCache fake;
 
-          for (QStringList::ConstIterator it = list.begin(); it != list.end();
+          for (QStringList::ConstIterator it = list.constBegin(); it != list.constEnd();
                ++it)
           {
             fake.setUid((*it).toULong());
@@ -590,7 +591,7 @@ IMAP4Protocol::listDir (const KUrl & _url)
         if (mySection.isEmpty()) mySection = "UID RFC822.SIZE ENVELOPE";
 
         bool withFlags = mySection.toUpper().contains("FLAGS") ;
-        imapCommand *fetch =
+        CommandPtr fetch =
           sendCommand (imapCommand::
                        clientFetch (mySequence, mySection));
         imapCache *cache;
@@ -632,9 +633,9 @@ IMAP4Protocol::setHost (const QString & _host, quint16 _port,
     if (!myHost.isEmpty ())
       closeConnection ();
     myHost = _host;
-    if (_port == 0) 
-        myPort = (mySSL) ? IMAPS_PORT : IMAP_PORT;
-    else 
+    if (_port == 0)
+        myPort = (mySSL) ? ImapsPort : ImapPort;
+    else
         myPort = _port;
     myUser = _user;
     myPass = _pass;
@@ -729,12 +730,9 @@ bool IMAP4Protocol::parseReadLine (QByteArray & buffer, long relay)
       }
       // append to buffer
       {
-        QBuffer stream (&buffer);
-
-        stream.open (QIODevice::WriteOnly);
-        stream.seek (buffer.size ());
-        stream.write (readBuffer, copyLen);
-        stream.close ();
+        int oldsize = buffer.size();
+        buffer.resize(oldsize + copyLen);
+        memcpy(buffer.data() + oldsize, readBuffer, copyLen);
 //        kDebug(7116) <<"appended" << copyLen <<"d got now" << buffer.size();
       }
 
@@ -791,7 +789,7 @@ IMAP4Protocol::put (const KUrl & _url, int, KIO::JobFlags)
   {
     if (aBox[aBox.length () - 1] == '/')
       aBox = aBox.right (aBox.length () - 1);
-    imapCommand *cmd = doCommand (imapCommand::clientCreate (aBox));
+    CommandPtr cmd = doCommand (imapCommand::clientCreate (aBox));
 
     if (cmd->result () != "OK") {
       error (ERR_COULD_NOT_WRITE, _url.prettyUrl());
@@ -828,7 +826,7 @@ IMAP4Protocol::put (const KUrl & _url, int, KIO::JobFlags)
       return;
     }
 
-    imapCommand *cmd =
+    CommandPtr cmd =
       sendCommand (imapCommand::clientAppend (aBox, aSection, length));
     while (!parseLoop ()) {}
 
@@ -884,7 +882,7 @@ IMAP4Protocol::put (const KUrl & _url, int, KIO::JobFlags)
           QString uid = cmd->resultInfo();
           if ( uid.contains("APPENDUID") )
           {
-            uid = uid.section(" ", 2, 2);
+            uid = uid.section(' ', 2, 2);
             uid.truncate(uid.length()-1);
             infoMessage("UID "+uid);
           }
@@ -921,7 +919,7 @@ IMAP4Protocol::mkdir (const KUrl & _url, int)
   QString aBox, aSequence, aLType, aSection, aValidity, aDelimiter, aInfo;
   parseURL(_url, aBox, aSection, aLType, aSequence, aValidity, aDelimiter, aInfo);
   kDebug(7116) <<"IMAP4::mkdir - create" << aBox;
-  imapCommand *cmd = doCommand (imapCommand::clientCreate(aBox));
+  CommandPtr cmd = doCommand (imapCommand::clientCreate(aBox));
 
   if (cmd->result () != "OK")
   {
@@ -1013,7 +1011,7 @@ IMAP4Protocol::copy (const KUrl & src, const KUrl & dest, int, KIO::JobFlags fla
         if (dType != ITYPE_BOX && dType != ITYPE_DIR_AND_BOX)
         {
           // ok then we'll create a mailbox
-          imapCommand *cmd = doCommand (imapCommand::clientCreate (topDir));
+          CommandPtr cmd = doCommand (imapCommand::clientCreate (topDir));
 
           // on success we'll use it, else we'll just try to create the given dir
           if (cmd->result () == "OK")
@@ -1044,7 +1042,7 @@ IMAP4Protocol::copy (const KUrl & src, const KUrl & dest, int, KIO::JobFlags fla
     kDebug(7116) <<"IMAP4::copy -" << sBox <<" ->" << dBox;
 
     //issue copy command
-    imapCommand *cmd =
+    CommandPtr cmd =
       doCommand (imapCommand::clientCopy (dBox, sSequence));
     if (cmd->result () != "OK")
     {
@@ -1058,7 +1056,7 @@ IMAP4Protocol::copy (const KUrl & src, const KUrl & dest, int, KIO::JobFlags fla
         QString uid = cmd->resultInfo();
         if ( uid.contains("COPYUID") )
         {
-          uid = uid.section(" ", 2, 3);
+          uid = uid.section(' ', 2, 3);
           uid.truncate(uid.length()-1);
           infoMessage("UID "+uid);
         }
@@ -1091,7 +1089,7 @@ IMAP4Protocol::del (const KUrl & _url, bool isFile)
       if (aSequence == "*")
       {
         if (!assureBox (aBox, false)) return;
-        imapCommand *cmd = doCommand (imapCommand::clientExpunge ());
+        CommandPtr cmd = doCommand (imapCommand::clientExpunge ());
         if (cmd->result () != "OK") {
           error (ERR_CANNOT_DELETE, _url.prettyUrl());
           completeQueue.removeAll (cmd);
@@ -1103,7 +1101,7 @@ IMAP4Protocol::del (const KUrl & _url, bool isFile)
       {
         // if open for read/write
         if (!assureBox (aBox, false)) return;
-        imapCommand *cmd =
+        CommandPtr cmd =
           doCommand (imapCommand::
                      clientStore (aSequence, "+FLAGS.SILENT", "\\DELETED"));
         if (cmd->result () != "OK") {
@@ -1118,12 +1116,12 @@ IMAP4Protocol::del (const KUrl & _url, bool isFile)
     {
       if (getCurrentBox() == aBox)
       {
-        imapCommand *cmd = doCommand(imapCommand::clientClose());
+        CommandPtr cmd = doCommand(imapCommand::clientClose());
         completeQueue.removeAll(cmd);
         setState(ISTATE_LOGIN);
       }
       // We unsubscribe, otherwise we get ghost folders on UW-IMAP
-      imapCommand *cmd = doCommand(imapCommand::clientUnsubscribe(aBox));
+      CommandPtr cmd = doCommand(imapCommand::clientUnsubscribe(aBox));
       completeQueue.removeAll(cmd);
       cmd = doCommand(imapCommand::clientDelete (aBox));
       // If this doesn't work, we try to empty the mailbox first
@@ -1134,21 +1132,21 @@ IMAP4Protocol::del (const KUrl & _url, bool isFile)
         bool stillOk = true;
         if (stillOk)
         {
-          imapCommand *cmd = doCommand(
+          CommandPtr cmd = doCommand(
             imapCommand::clientStore("1:*", "+FLAGS.SILENT", "\\DELETED"));
           if (cmd->result () != "OK") stillOk = false;
           completeQueue.removeAll(cmd);
         }
         if (stillOk)
         {
-          imapCommand *cmd = doCommand(imapCommand::clientClose());
+          CommandPtr cmd = doCommand(imapCommand::clientClose());
           if (cmd->result () != "OK") stillOk = false;
           completeQueue.removeAll(cmd);
           setState(ISTATE_LOGIN);
         }
         if (stillOk)
         {
-          imapCommand *cmd = doCommand (imapCommand::clientDelete(aBox));
+          CommandPtr cmd = doCommand (imapCommand::clientDelete(aBox));
           if (cmd->result () != "OK") stillOk = false;
           completeQueue.removeAll(cmd);
         }
@@ -1165,7 +1163,7 @@ IMAP4Protocol::del (const KUrl & _url, bool isFile)
 
   case ITYPE_DIR:
     {
-      imapCommand *cmd = doCommand (imapCommand::clientDelete (aBox));
+      CommandPtr cmd = doCommand (imapCommand::clientDelete (aBox));
       if (cmd->result () != "OK") {
         error (ERR_COULD_NOT_RMDIR, _url.prettyUrl());
         completeQueue.removeAll (cmd);
@@ -1179,7 +1177,7 @@ IMAP4Protocol::del (const KUrl & _url, bool isFile)
     {
       // if open for read/write
       if (!assureBox (aBox, false)) return;
-      imapCommand *cmd =
+      CommandPtr cmd =
         doCommand (imapCommand::
                    clientStore (aSequence, "+FLAGS.SILENT", "\\DELETED"));
       if (cmd->result () != "OK") {
@@ -1246,7 +1244,7 @@ IMAP4Protocol::special (const QByteArray & aData)
   case 'N':
   {
     // NOOP
-    imapCommand *cmd = doCommand(imapCommand::clientNoop());
+    CommandPtr cmd = doCommand(imapCommand::clientNoop());
     if (cmd->result () != "OK")
     {
       kDebug(7116) <<"NOOP did not succeed - connection broken";
@@ -1255,7 +1253,6 @@ IMAP4Protocol::special (const QByteArray & aData)
       return;
     }
     completeQueue.removeAll (cmd);
-    delete cmd;
     finished();
     break;
   }
@@ -1274,7 +1271,7 @@ IMAP4Protocol::special (const QByteArray & aData)
     stream >> _url;
     QString aBox, aSequence, aLType, aSection, aValidity, aDelimiter, aInfo;
     parseURL (_url, aBox, aSection, aLType, aSequence, aValidity, aDelimiter, aInfo);
-    imapCommand *cmd = doCommand(imapCommand::clientUnsubscribe(aBox));
+    CommandPtr cmd = doCommand(imapCommand::clientUnsubscribe(aBox));
     if (cmd->result () != "OK")
     {
       completeQueue.removeAll (cmd);
@@ -1295,7 +1292,7 @@ IMAP4Protocol::special (const QByteArray & aData)
     stream >> _url;
     QString aBox, aSequence, aLType, aSection, aValidity, aDelimiter, aInfo;
     parseURL (_url, aBox, aSection, aLType, aSequence, aValidity, aDelimiter, aInfo);
-    imapCommand *cmd = doCommand(imapCommand::clientSubscribe(aBox));
+    CommandPtr cmd = doCommand(imapCommand::clientSubscribe(aBox));
     if (cmd->result () != "OK")
     {
       completeQueue.removeAll (cmd);
@@ -1317,7 +1314,7 @@ IMAP4Protocol::special (const QByteArray & aData)
     if ( hasCapability( "ACL" ) ) {
       specialACLCommand( cmd, stream );
     } else {
-      error( ERR_UNSUPPORTED_ACTION, "ACL" );
+      error( ERR_UNSUPPORTED_ACTION, QString::fromLatin1("ACL") );
     }
     break;
   }
@@ -1329,7 +1326,7 @@ IMAP4Protocol::special (const QByteArray & aData)
     if ( hasCapability( "ANNOTATEMORE" ) ) {
       specialAnnotateMoreCommand( cmd, stream );
     } else {
-      error( ERR_UNSUPPORTED_ACTION, "ANNOTATEMORE" );
+      error( ERR_UNSUPPORTED_ACTION, QString::fromLatin1("ANNOTATEMORE") );
     }
     break;
   }
@@ -1341,7 +1338,7 @@ IMAP4Protocol::special (const QByteArray & aData)
     if ( hasCapability( "QUOTA" ) ) {
       specialQuotaCommand( cmd, stream );
     } else {
-      error( ERR_UNSUPPORTED_ACTION, "QUOTA" );
+      error( ERR_UNSUPPORTED_ACTION, QString::fromLatin1("QUOTA") );
     }
     break;
   }
@@ -1363,8 +1360,8 @@ IMAP4Protocol::special (const QByteArray & aData)
       knownFlags += " KMAILFORWARDED KMAILTODO KMAILWATCHED KMAILIGNORED $FORWARDED $TODO $WATCHED $IGNORED";
     }
 
-    imapCommand *cmd = doCommand (imapCommand::
-                                  clientStore (aSequence, "-FLAGS.SILENT", knownFlags));
+    CommandPtr cmd = doCommand (imapCommand::
+                                clientStore (aSequence, "-FLAGS.SILENT", knownFlags));
     if (cmd->result () != "OK")
     {
       completeQueue.removeAll (cmd);
@@ -1402,7 +1399,7 @@ IMAP4Protocol::special (const QByteArray & aData)
     if ( !assureBox(aBox, true) ) // read-only because changing SEEN should be possible even then
       return;
 
-    imapCommand *cmd;
+    CommandPtr cmd;
     if ( seen )
       cmd = doCommand( imapCommand::clientStore( aSequence, "+FLAGS.SILENT", "\\SEEN" ) );
     else
@@ -1411,7 +1408,7 @@ IMAP4Protocol::special (const QByteArray & aData)
     if (cmd->result () != "OK")
     {
       completeQueue.removeAll (cmd);
-      error(ERR_COULD_NOT_WRITE,
+      error(ERR_SLAVE_DEFINED,
             i18n( "Changing the flags of message %1 failed.", _url.prettyUrl() ) );
       return;
     }
@@ -1454,7 +1451,7 @@ IMAP4Protocol::specialACLCommand( int command, QDataStream& stream )
     QString user, acl;
     stream >> user >> acl;
     kDebug(7116) <<"SETACL" << aBox << user << acl;
-    imapCommand *cmd = doCommand(imapCommand::clientSetACL(aBox, user, acl));
+    CommandPtr cmd = doCommand(imapCommand::clientSetACL(aBox, user, acl));
     if (cmd->result () != "OK")
     {
       error(ERR_SLAVE_DEFINED, i18n("Setting the Access Control List on folder %1 "
@@ -1473,7 +1470,7 @@ IMAP4Protocol::specialACLCommand( int command, QDataStream& stream )
     QString user;
     stream >> user;
     kDebug(7116) <<"DELETEACL" << aBox << user;
-    imapCommand *cmd = doCommand(imapCommand::clientDeleteACL(aBox, user));
+    CommandPtr cmd = doCommand(imapCommand::clientDeleteACL(aBox, user));
     if (cmd->result () != "OK")
     {
       error(ERR_SLAVE_DEFINED, i18n("Deleting the Access Control List on folder %1 "
@@ -1490,7 +1487,7 @@ IMAP4Protocol::specialACLCommand( int command, QDataStream& stream )
   case 'G': // GETACL
   {
     kDebug(7116) <<"GETACL" << aBox;
-    imapCommand *cmd = doCommand(imapCommand::clientGetACL(aBox));
+    CommandPtr cmd = doCommand(imapCommand::clientGetACL(aBox));
     if (cmd->result () != "OK")
     {
       error(ERR_SLAVE_DEFINED, i18n("Retrieving the Access Control List on folder %1 "
@@ -1501,10 +1498,10 @@ IMAP4Protocol::specialACLCommand( int command, QDataStream& stream )
     }
     // Returning information to the application from a special() command isn't easy.
     // I'm reusing the infoMessage trick seen above (for capabilities), but this
-    // limits me to a string instead of a stringlist. I'm using space as separator,
-    // since I don't think it can be used in login names.
+    // limits me to a string instead of a stringlist. Using DQUOTE as separator,
+    // because it's forbidden in userids by rfc3501
     kDebug(7116) << getResults();
-    infoMessage(getResults().join( " " ));
+    infoMessage(getResults().join( "\"" ));
     finished();
     break;
   }
@@ -1517,7 +1514,7 @@ IMAP4Protocol::specialACLCommand( int command, QDataStream& stream )
   case 'M': // MYRIGHTS
   {
     kDebug(7116) <<"MYRIGHTS" << aBox;
-    imapCommand *cmd = doCommand(imapCommand::clientMyRights(aBox));
+    CommandPtr cmd = doCommand(imapCommand::clientMyRights(aBox));
     if (cmd->result () != "OK")
     {
       error(ERR_SLAVE_DEFINED, i18n("Retrieving the Access Control List on folder %1 "
@@ -1549,9 +1546,9 @@ IMAP4Protocol::specialSearchCommand( QDataStream& stream )
   stream >> _url;
   QString aBox, aSequence, aLType, aSection, aValidity, aDelimiter, aInfo;
   parseURL (_url, aBox, aSection, aLType, aSequence, aValidity, aDelimiter, aInfo);
-  if (!assureBox(aBox, false)) return;
+  if (!assureBox(aBox, true)) return;
 
-  imapCommand *cmd = doCommand (imapCommand::clientSearch( aSection ));
+  CommandPtr cmd = doCommand (imapCommand::clientSearch( aSection ));
   if (cmd->result () != "OK")
   {
     error(ERR_SLAVE_DEFINED, i18n("Searching of folder %1 "
@@ -1585,14 +1582,12 @@ IMAP4Protocol::specialCustomCommand( QDataStream& stream )
    */
   if ( type == 'N' ) {
     kDebug(7116) << "IMAP4Protocol::specialCustomCommand: normal mode" << endl;
-    imapCommand *cmd = doCommand (imapCommand::clientCustom( command, arguments ));
+    CommandPtr cmd = doCommand (imapCommand::clientCustom( command, arguments ));
     if (cmd->result () != "OK")
     {
-      error(ERR_SLAVE_DEFINED, i18n("Custom command %1:%2 "
-            "failed. The server returned: %3")
-          .arg(command)
-          .arg(arguments)
-          .arg(cmd->resultInfo()));
+      error( ERR_SLAVE_DEFINED,
+             i18n( "Custom command %1:%2 failed. The server returned: %3",
+                  command, arguments, cmd->resultInfo() ) );
       return;
     }
     completeQueue.removeAll(cmd);
@@ -1610,8 +1605,8 @@ IMAP4Protocol::specialCustomCommand( QDataStream& stream )
    */
   if ( type == 'E' ) {
     kDebug(7116) << "IMAP4Protocol::specialCustomCommand: extended mode" << endl;
-    imapCommand *cmd = sendCommand (imapCommand::clientCustom( command, QString() ));
-    while ( !parseLoop () );
+    CommandPtr cmd = sendCommand (imapCommand::clientCustom( command, QString() ));
+    while ( !parseLoop () ) {};
 
     // see if server is waiting
     if (!cmd->isComplete () && !getContinuation ().isEmpty ())
@@ -1634,7 +1629,7 @@ IMAP4Protocol::specialCustomCommand( QDataStream& stream )
 
     do
     {
-      while (!parseLoop ());
+      while (!parseLoop ()) {};
     }
     while (!cmd->isComplete ());
 
@@ -1668,7 +1663,7 @@ IMAP4Protocol::specialAnnotateMoreCommand( int command, QDataStream& stream )
     QMap<QString, QString> attributes;
     stream >> entry >> attributes;
     kDebug(7116) <<"SETANNOTATION" << aBox << entry << attributes.count() <<" attributes";
-    imapCommand *cmd = doCommand(imapCommand::clientSetAnnotation(aBox, entry, attributes));
+    CommandPtr cmd = doCommand(imapCommand::clientSetAnnotation(aBox, entry, attributes));
     if (cmd->result () != "OK")
     {
       error(ERR_SLAVE_DEFINED, i18n("Setting the annotation %1 on folder %2 "
@@ -1692,7 +1687,7 @@ IMAP4Protocol::specialAnnotateMoreCommand( int command, QDataStream& stream )
     QStringList attributeNames;
     stream >> entry >> attributeNames;
     kDebug(7116) <<"GETANNOTATION" << aBox << entry << attributeNames;
-    imapCommand *cmd = doCommand(imapCommand::clientGetAnnotation(aBox, entry, attributeNames));
+    CommandPtr cmd = doCommand(imapCommand::clientGetAnnotation(aBox, entry, attributeNames));
     if (cmd->result () != "OK")
     {
       error(ERR_SLAVE_DEFINED, i18n("Retrieving the annotation %1 on folder %2 "
@@ -1729,7 +1724,7 @@ IMAP4Protocol::specialQuotaCommand( int command, QDataStream& stream )
     case 'R': // GETQUOTAROOT
       {
         kDebug(7116) <<"QUOTAROOT" << aBox;
-        imapCommand *cmd = doCommand(imapCommand::clientGetQuotaroot( aBox ) );
+        CommandPtr cmd = doCommand(imapCommand::clientGetQuotaroot( aBox ) );
         if (cmd->result () != "OK")
         {
           error(ERR_SLAVE_DEFINED, i18n("Retrieving the quota root information on folder %1 "
@@ -1783,17 +1778,17 @@ IMAP4Protocol::rename (const KUrl & src, const KUrl & dest, KIO::JobFlags flags)
         {
           kDebug(7116) <<"IMAP4::rename - close" << getCurrentBox();
           // mailbox can only be renamed if it is closed
-          imapCommand *cmd = doCommand (imapCommand::clientClose());
+          CommandPtr cmd = doCommand (imapCommand::clientClose());
           bool ok = cmd->result() == "OK";
           completeQueue.removeAll(cmd);
           if (!ok)
           {
-            error(ERR_CANNOT_RENAME, i18n("Unable to close mailbox."));
+            error(ERR_SLAVE_DEFINED, i18n("Unable to close mailbox."));
             return;
           }
           setState(ISTATE_LOGIN);
         }
-        imapCommand *cmd = doCommand (imapCommand::clientRename (sBox, dBox));
+        CommandPtr cmd = doCommand (imapCommand::clientRename (sBox, dBox));
         if (cmd->result () != "OK") {
           error (ERR_CANNOT_RENAME, cmd->result ());
           completeQueue.removeAll (cmd);
@@ -1851,12 +1846,12 @@ IMAP4Protocol::stat (const KUrl & _url)
   {
     if (getState() == ISTATE_SELECT && aBox == getCurrentBox())
     {
-      imapCommand *cmd = doCommand (imapCommand::clientClose());
+      CommandPtr cmd = doCommand (imapCommand::clientClose());
       bool ok = cmd->result() == "OK";
       completeQueue.removeAll(cmd);
       if (!ok)
       {
-        error(ERR_COULD_NOT_STAT, i18n("Unable to close mailbox."));
+        error(ERR_SLAVE_DEFINED, i18n("Unable to close mailbox."));
         return;
       }
       setState(ISTATE_LOGIN);
@@ -1867,7 +1862,7 @@ IMAP4Protocol::stat (const KUrl & _url)
       ok = true;
     else
     {
-      imapCommand *cmd = doCommand(imapCommand::clientStatus(aBox, aSection));
+      CommandPtr cmd = doCommand(imapCommand::clientStatus(aBox, aSection));
       ok = cmd->result() == "OK";
       cmdInfo = cmd->resultInfo();
       completeQueue.removeAll(cmd);
@@ -1875,7 +1870,7 @@ IMAP4Protocol::stat (const KUrl & _url)
     if (!ok)
     {
       bool found = false;
-      imapCommand *cmd = doCommand (imapCommand::clientList ("", aBox));
+      CommandPtr cmd = doCommand (imapCommand::clientList ("", aBox));
       if (cmd->result () == "OK")
       {
         for (QList< imapList >::Iterator it = listResponses.begin ();
@@ -1886,7 +1881,7 @@ IMAP4Protocol::stat (const KUrl & _url)
       }
       completeQueue.removeAll (cmd);
       if (found)
-        error(ERR_COULD_NOT_STAT, i18n("Unable to get information about folder %1. The server replied: %2", aBox, cmdInfo));
+        error(ERR_SLAVE_DEFINED, i18n("Unable to get information about folder %1. The server replied: %2", aBox, cmdInfo));
       else
         error(KIO::ERR_DOES_NOT_EXIST, aBox);
       return;
@@ -1910,7 +1905,7 @@ IMAP4Protocol::stat (const KUrl & _url)
       // do a status lookup on the box
       // only do this if the box is not selected
       // the server might change the validity for new select/examine
-      imapCommand *cmd =
+      CommandPtr cmd =
         doCommand (imapCommand::clientStatus (aBox, "UIDVALIDITY"));
       completeQueue.removeAll (cmd);
       validity = getStatus ().uidValidity ();
@@ -1988,12 +1983,12 @@ void IMAP4Protocol::closeConnection()
   if (getState() == ISTATE_NO) return;
   if (getState() == ISTATE_SELECT && metaData("expunge") == "auto")
   {
-    imapCommand *cmd = doCommand (imapCommand::clientExpunge());
+    CommandPtr cmd = doCommand (imapCommand::clientExpunge());
     completeQueue.removeAll (cmd);
   }
   if (getState() != ISTATE_CONNECT)
   {
-    imapCommand *cmd = doCommand (imapCommand::clientLogout());
+    CommandPtr cmd = doCommand (imapCommand::clientLogout());
     completeQueue.removeAll (cmd);
   }
   disconnectFromHost();
@@ -2024,23 +2019,22 @@ bool IMAP4Protocol::makeLogin ()
     myTLS  = metaData("tls");
     kDebug(7116) <<"myAuth:" << myAuth;
 
-    imapCommand *cmd;
+    CommandPtr cmd;
 
     unhandled.clear ();
     if (!alreadyConnected) while (!parseLoop ()) {}   //get greeting
     QString greeting;
     if (!unhandled.isEmpty()) greeting = unhandled.first().trimmed();
     unhandled.clear ();       //get rid of it
-    cmd = doCommand (new imapCommand ("CAPABILITY", ""));
+    cmd = doCommand (CommandPtr(new imapCommand ("CAPABILITY", "")));
 
     kDebug(7116) <<"IMAP4: setHost: capability";
-    for (QStringList::Iterator it = imapCapabilities.begin ();
-         it != imapCapabilities.end (); ++it)
+    for (QStringList::const_iterator it = imapCapabilities.constBegin ();
+         it != imapCapabilities.constEnd (); ++it)
     {
       kDebug(7116) <<"'" << (*it) <<"'";
     }
     completeQueue.removeAll (cmd);
-    delete cmd;
 
     if (!hasCapability("IMAP4") && !hasCapability("IMAP4rev1"))
     {
@@ -2063,16 +2057,16 @@ bool IMAP4Protocol::makeLogin ()
     if ((myTLS == "on" /*###|| ( canUseTLS() && myTLS != "off")*/) &&
         hasCapability(QString("STARTTLS")))
     {
-      imapCommand *cmd = doCommand (imapCommand::clientStartTLS());
+      CommandPtr cmd = doCommand (imapCommand::clientStartTLS());
       if (cmd->result () == "OK")
       {
         completeQueue.removeAll(cmd);
         if (startSsl())
         {
           kDebug(7116) <<"TLS mode has been enabled.";
-          imapCommand *cmd2 = doCommand (new imapCommand ("CAPABILITY", ""));
-          for (QStringList::Iterator it = imapCapabilities.begin ();
-                                     it != imapCapabilities.end (); ++it)
+          CommandPtr cmd2 = doCommand (CommandPtr(new imapCommand ("CAPABILITY", "")));
+          for (QStringList::const_iterator it = imapCapabilities.constBegin ();
+                                     it != imapCapabilities.constEnd (); ++it)
           {
             kDebug(7116) <<"'" << (*it) <<"'";
           }
@@ -2081,11 +2075,9 @@ bool IMAP4Protocol::makeLogin ()
           kWarning(7116) <<"TLS mode setup has failed.  Aborting.";
           error (ERR_COULD_NOT_LOGIN, i18n("Starting TLS failed."));
           closeConnection();
-          delete cmd;
           return false;
         }
       } else completeQueue.removeAll(cmd);
-      delete cmd;
     }
 
     if (!myAuth.isEmpty () && myAuth != "*"
@@ -2101,6 +2093,18 @@ bool IMAP4Protocol::makeLogin ()
       removeCapability( "ANNOTATEMORE" );
     }
 
+    // starting from Cyrus IMAP 2.3.9, shared seen flags are available
+    QRegExp regExp( "Cyrus\\sIMAP[4]{0,1}\\sv(\\d+)\\.(\\d+)\\.(\\d+)", Qt::CaseInsensitive );
+    if ( regExp.indexIn( greeting ) >= 0 ) {
+      const int major = regExp.cap( 1 ).toInt();
+      const int minor = regExp.cap( 2 ).toInt();
+      const int patch = regExp.cap( 3 ).toInt();
+      if ( major > 2 || (major == 2 && (minor > 3 || (minor == 3 && patch > 9))) ) {
+        kDebug(7116) << "Cyrus IMAP >= 2.3.9 detected, enabling shared seen flag support";
+        imapCapabilities.append( "x-kmail-sharedseen" );
+      }
+    }
+
     kDebug(7116) <<"IMAP4::makeLogin - attempting login";
 
     KIO::AuthInfo authInfo;
@@ -2120,21 +2124,17 @@ bool IMAP4Protocol::makeLogin ()
         }
       }
       if (!clientLogin (myUser, myPass, resultInfo))
-        error(KIO::ERR_COULD_NOT_AUTHENTICATE, i18n("Unable to login. Probably the "
+        error(ERR_SLAVE_DEFINED, i18n("Unable to login. Probably the "
         "password is wrong.\nThe server %1 replied:\n%2", myHost, resultInfo));
     }
     else
     {
-#ifdef HAVE_LIBSASL2
       if (!clientAuthenticate (this, authInfo, myHost, myAuth, mySSL, resultInfo))
-        error(KIO::ERR_COULD_NOT_AUTHENTICATE, i18n("Unable to authenticate via %1.\n"	"The server %2 replied:\n%3", myAuth, myHost, resultInfo));
+        error(ERR_SLAVE_DEFINED, i18n("Unable to authenticate via %1.\n"	"The server %2 replied:\n%3", myAuth, myHost, resultInfo));
       else {
         myUser = authInfo.username;
         myPass = authInfo.password;
       }
-#else
-      error(KIO::ERR_COULD_NOT_LOGIN, i18n("SASL authentication is not compiled into kio_imap4."));
-#endif
     }
     if ( hasCapability("NAMESPACE") )
     {
@@ -2145,7 +2145,6 @@ bool IMAP4Protocol::makeLogin ()
         kDebug(7116) <<"makeLogin - registered namespaces";
       }
       completeQueue.removeAll (cmd);
-      delete cmd;
     }
     // get the default delimiter (empty listing)
     cmd = doCommand( imapCommand::clientList("", "") );
@@ -2165,7 +2164,6 @@ bool IMAP4Protocol::makeLogin ()
       }
     }
     completeQueue.removeAll (cmd);
-    delete cmd;
   } else {
     kDebug(7116) <<"makeLogin - NO login";
   }
@@ -2417,7 +2415,7 @@ IMAP4Protocol::parseURL (const KUrl & _url, QString & _box,
         } else
         {
           // start a listing for the box to get the type
-          imapCommand *cmd;
+          CommandPtr cmd;
 
           cmd = doCommand (imapCommand::clientList ("", _box));
           if (cmd->result () == "OK")
@@ -2502,7 +2500,7 @@ IMAP4Protocol::parseURL (const KUrl & _url, QString & _box,
         << "from URL" << _url.path();
     }
     if (_hierarchyDelimiter.isEmpty())
-      _hierarchyDelimiter = "/";
+      _hierarchyDelimiter = '/';
   }
   kDebug(7116) <<"IMAP4::parseURL - return" << retVal;
 
@@ -2550,9 +2548,9 @@ void IMAP4Protocol::flushOutput(const QString &contentEncoding)
   {
     // get the coding from the MIME header
     QByteArray decoded;
-    if ( contentEncoding.startsWith("quoted-printable", Qt::CaseInsensitive) )
+    if ( contentEncoding.startsWith(QLatin1String("quoted-printable"), Qt::CaseInsensitive) )
       decoded = KCodecs::quotedPrintableDecode(outputCache);
-    else if ( contentEncoding.startsWith("base64", Qt::CaseInsensitive) )
+    else if ( contentEncoding.startsWith(QLatin1String("base64"), Qt::CaseInsensitive) )
       decoded = QByteArray::fromBase64( outputCache );
     else
       decoded = outputCache;
@@ -2592,7 +2590,7 @@ IMAP4Protocol::assureBox (const QString & aBox, bool readonly)
 {
   if (aBox.isEmpty()) return false;
 
-  imapCommand *cmd = 0;
+  CommandPtr cmd;
 
   if (aBox != getCurrentBox () || (!getSelected().readWrite() && !readonly))
   {
