@@ -34,11 +34,9 @@
 #include <unistd.h>
 #include <QList>
 
-#ifdef HAVE_LIBSASL2
 extern "C" {
 #include <sasl/sasl.h>
 }
-#endif
 
 #include <QRegExp>
 #include <QBuffer>
@@ -54,7 +52,6 @@ extern "C" {
 #include <kimap/rfccodecs.h>
 using namespace KIMAP;
 
-#ifdef HAVE_LIBSASL2
 static sasl_callback_t callbacks[] = {
     { SASL_CB_ECHOPROMPT, NULL, NULL },
     { SASL_CB_NOECHOPROMPT, NULL, NULL },
@@ -65,7 +62,6 @@ static sasl_callback_t callbacks[] = {
     { SASL_CB_CANON_USER, NULL, NULL },
     { SASL_CB_LIST_END, NULL, NULL }
 };
-#endif
 
 imapParser::imapParser ()
 {
@@ -80,8 +76,8 @@ imapParser::~imapParser ()
   lastHandled = 0;
 }
 
-imapCommand *
-imapParser::doCommand (imapCommand * aCmd)
+CommandPtr
+imapParser::doCommand (CommandPtr aCmd)
 {
   int pl = 0;
   sendCommand (aCmd);
@@ -93,8 +89,8 @@ imapParser::doCommand (imapCommand * aCmd)
   return aCmd;
 }
 
-imapCommand *
-imapParser::sendCommand (imapCommand * aCmd)
+CommandPtr
+imapParser::sendCommand (CommandPtr aCmd)
 {
   aCmd->setId (QString::number(commandCounter++));
   sentQueue.append (aCmd);
@@ -142,13 +138,13 @@ bool
 imapParser::clientLogin (const QString & aUser, const QString & aPass,
   QString & resultInfo)
 {
-  imapCommand *cmd;
+  CommandPtr cmd;
   bool retVal = false;
 
   cmd =
-    doCommand (new
-               imapCommand ("LOGIN", "\"" + KIMAP::quoteIMAP(aUser)
-               + "\" \"" + KIMAP::quoteIMAP(aPass) + "\""));
+    doCommand ( CommandPtr( new
+                            imapCommand ("LOGIN", "\"" + KIMAP::quoteIMAP(aUser)
+                                         + "\" \"" + KIMAP::quoteIMAP(aPass) + "\"")) );
 
   if (cmd->result () == "OK")
   {
@@ -157,11 +153,9 @@ imapParser::clientLogin (const QString & aUser, const QString & aPass,
   }
   resultInfo = cmd->resultInfo();
   completeQueue.removeAll (cmd);
-  delete cmd;
   return retVal;
 }
 
-#ifdef HAVE_LIBSASL2
 static bool sasl_interact( KIO::SlaveBase *slave, KIO::AuthInfo &ai, void *in )
 {
   kDebug(7116) <<"sasl_interact";
@@ -205,14 +199,12 @@ static bool sasl_interact( KIO::SlaveBase *slave, KIO::AuthInfo &ai, void *in )
   }
   return true;
 }
-#endif
 
 bool
 imapParser::clientAuthenticate ( KIO::SlaveBase *slave, KIO::AuthInfo &ai,
   const QString & aFQDN, const QString & aAuth, bool isSSL, QString & resultInfo)
 {
   bool retVal = false;
-#ifdef HAVE_LIBSASL2
   int result;
   sasl_conn_t *conn = 0;
   sasl_interact_t *client_interact = 0;
@@ -257,7 +249,7 @@ imapParser::clientAuthenticate ( KIO::SlaveBase *slave, KIO::AuthInfo &ai,
     sasl_dispose( &conn );
     return false;
   }
-  imapCommand *cmd;
+  CommandPtr cmd;
 
   tmp = QByteArray::fromRawData( out, outlen );
   challenge = tmp.toBase64();
@@ -268,15 +260,14 @@ imapParser::clientAuthenticate ( KIO::SlaveBase *slave, KIO::AuthInfo &ai,
     firstCommand += ' ';
     firstCommand += QString::fromLatin1( challenge.data(), challenge.size() );
   }
-  cmd = sendCommand (new imapCommand ("AUTHENTICATE", firstCommand.toLatin1()));
+  cmd = sendCommand (CommandPtr(new imapCommand ("AUTHENTICATE", firstCommand.toLatin1())));
 
-  while ( true )
-  {
+  int pl = 0;
+  while ( pl != -1 && !cmd->isComplete () ) {
     //read the next line
-    while (parseLoop() == 0) {
+    while ( ( pl = parseLoop() ) == 0) {
       ;
     }
-    if ( cmd->isComplete() ) break;
 
     if (!continuation.isEmpty())
     {
@@ -328,7 +319,6 @@ imapParser::clientAuthenticate ( KIO::SlaveBase *slave, KIO::AuthInfo &ai,
   completeQueue.removeAll (cmd);
 
   sasl_dispose( &conn ); //we don't use sasl_en/decode(), so it's safe to dispose the connection.
-#endif //HAVE_LIBSASL2
   return retVal;
 }
 
@@ -353,7 +343,7 @@ imapParser::parseUntagged (parseString & result)
       parseResult (what, result);
       if ( sentQueue.count() ) {
         // BYE that interrupts a command -> copy the reason for it
-        imapCommand *current = sentQueue.at (0);
+        CommandPtr current = sentQueue.at (0);
         current->setResultInfo(result.cstr());
       }
       currentState = ISTATE_NO;
@@ -694,7 +684,7 @@ void imapParser::parseList (parseString & result)
   skipWS (result);
 
   this_one.setHierarchyDelimiter(parseLiteral(result));
-  this_one.setName (KIMAP::decodeImapFolderName( parseLiteral(result)));  // decode modified UTF7
+  this_one.setName(QString::fromUtf8(KIMAP::decodeImapFolderName( parseLiteral(result))));  // decode modified UTF7
 
   listResponses.append (this_one);
 }
@@ -789,7 +779,7 @@ void imapParser::parseQuotaRoot (parseString & result)
       break;
     roots.append (word);
   }
-  lastResults.append( roots.join(" ") );
+  lastResults.append( roots.isEmpty() ? "" : roots.join( " " ) );
 }
 
 void imapParser::parseCustom (parseString & result)
@@ -813,7 +803,7 @@ void imapParser::parseDelegate (parseString & result)
     rights.append( word );
   }
 
-  lastResults.append( email + ":" + rights.join( "," ) );
+  lastResults.append( email + ':' + rights.join( "," ) );
 }
 
 void imapParser::parseOutOfOffice (parseString & result)
@@ -823,7 +813,7 @@ void imapParser::parseOutOfOffice (parseString & result)
 
   QByteArray msg = parseLiteral (result, false, false);
 
-  lastResults.append( state + "^" + QString::fromUtf8( msg ) );
+  lastResults.append( state + '^' + QString::fromUtf8( msg ) );
 }
 
 void imapParser::parseMyRights (parseString & result)
@@ -893,6 +883,8 @@ void imapParser::parseExpunge (ulong value, parseString & result)
 
 void imapParser::parseAddressList (parseString & inWords, QList<mailAddress *>& list)
 {
+  if ( inWords.isEmpty() )
+    return;
   if (inWords[0] != '(')
   {
     parseOneWord (inWords);     // parse NIL
@@ -913,7 +905,7 @@ void imapParser::parseAddressList (parseString & inWords, QList<mailAddress *>&
       }
     }
 
-    if (inWords[0] == ')')
+    if (!inWords.isEmpty() && inWords[0] == ')')
       inWords.pos++;
     skipWS (inWords);
   }
@@ -924,12 +916,12 @@ const mailAddress& imapParser::parseAddress (parseString & inWords, mailAddress&
   inWords.pos++;
   skipWS (inWords);
 
-  retVal.setFullName(KIMAP::quoteIMAP(parseLiteral(inWords)));
+  retVal.setFullName(parseLiteral(inWords));
   retVal.setCommentRaw(parseLiteral(inWords));
   retVal.setUser(parseLiteral(inWords));
   retVal.setHost(parseLiteral(inWords));
 
-  if (inWords[0] == ')')
+  if (!inWords.isEmpty() && inWords[0] == ')')
     inWords.pos++;
   skipWS (inWords);
 
@@ -1001,7 +993,7 @@ mailHeader * imapParser::parseEnvelope (parseString & inWords)
       parseLiteral (inWords);
   }
 
-  if (inWords[0] == ')')
+  if (!inWords.isEmpty() && inWords[0] == ')')
     inWords.pos++;
   skipWS (inWords);
 
@@ -1200,7 +1192,7 @@ mimeHeader * imapParser::parseBodyStructure (parseString & inWords,
     // first run
     init = true;
     // assume one part
-    inSection = "1";
+    inSection = '1';
   }
   int section = 0;
 
@@ -1231,7 +1223,7 @@ mimeHeader * imapParser::parseBodyStructure (parseString & inWords,
       outSection = inSection + ".HEADER";
     }
     if (inWords[0] == '(' && init)
-      inSection = "0";
+      inSection = '0';
 
     // set the section
     if ( !outSection.isEmpty() ) {
@@ -1240,7 +1232,7 @@ mimeHeader * imapParser::parseBodyStructure (parseString & inWords,
       localPart->setPartSpecifier(inSection);
     }
 
-    // is multipart (otherwise its a simplepart and handled later)
+    // is multipart (otherwise it is a simplepart and handled later)
     while (inWords[0] == '(')
     {
       outSection = QString::number(++section);
@@ -1515,7 +1507,8 @@ void imapParser::parseFetch (ulong /* value */, parseString & inWords)
             parseBodyStructure (inWords, section, envelope);
           QByteArray data;
           QDataStream stream( &data, QIODevice::WriteOnly );
-          body->serialize(stream);
+          if ( body )
+            body->serialize(stream);
           parseRelay(data);
 
           delete body;
@@ -1555,7 +1548,7 @@ void imapParser::parseFetch (ulong /* value */, parseString & inWords)
           if (!lastHandled) lastHandled = new imapCache();
           lastHandled->setSize (size);
         }
-        else if (word.startsWith("RFC822"))
+        else if (word.startsWith("RFC822")) //krazy:exclude=strings
         {
           // might be RFC822 RFC822.TEXT RFC822.HEADER
           parseLiteral(inWords, true);
@@ -1597,7 +1590,7 @@ void imapParser::parseFetch (ulong /* value */, parseString & inWords)
       parseLiteral(inWords);
   }
 
-  if (inWords[0] != ')')
+  if (inWords.isEmpty() || inWords[0] != ')')
     return;
   inWords.pos++;
   skipWS (inWords);
@@ -1742,7 +1735,7 @@ int imapParser::parseLoop ()
   }
   else
   {
-    imapCommand *current = sentQueue.at (0);
+    CommandPtr current = sentQueue.at (0);
     switch (result[0])
     {
     case '*':
@@ -1843,8 +1836,8 @@ imapParser::parseURL (const KUrl & _url, QString & _box, QString & _section,
     _box.truncate( paramStart ); // strip parameters
   }
   // extract parameters
-  for (QStringList::ConstIterator it (parameters.begin ());
-       it != parameters.end (); ++it)
+  for (QStringList::ConstIterator it (parameters.constBegin ());
+       it != parameters.constEnd (); ++it)
   {
     QString temp = (*it);
 
@@ -1852,15 +1845,15 @@ imapParser::parseURL (const KUrl & _url, QString & _box, QString & _section,
     int pt = temp.indexOf ('/');
     if (pt > 0)
       temp.truncate(pt);
-    if (temp.startsWith("section=", Qt::CaseInsensitive))
+    if (temp.startsWith(QLatin1String("section="), Qt::CaseInsensitive))
       _section = temp.right (temp.length () - 8);
-    else if (temp.startsWith("type=", Qt::CaseInsensitive))
+    else if (temp.startsWith(QLatin1String("type="), Qt::CaseInsensitive))
       _type = temp.right (temp.length () - 5);
-    else if (temp.startsWith("uid=", Qt::CaseInsensitive))
+    else if (temp.startsWith(QLatin1String("uid="), Qt::CaseInsensitive))
       _uid = temp.right (temp.length () - 4);
-    else if (temp.startsWith("uidvalidity=", Qt::CaseInsensitive))
+    else if (temp.startsWith(QLatin1String("uidvalidity="), Qt::CaseInsensitive))
       _validity = temp.right (temp.length () - 12);
-    else if (temp.startsWith("info=", Qt::CaseInsensitive))
+    else if (temp.startsWith(QLatin1String("info="), Qt::CaseInsensitive))
       _info = temp.right (temp.length () - 5);
   }
 //  kDebug(7116) <<"URL: section=" << _section <<", type=" << _type <<", uid=" << _uid;
@@ -1883,7 +1876,7 @@ imapParser::parseURL (const KUrl & _url, QString & _box, QString & _section,
 
 QByteArray imapParser::parseLiteral(parseString & inWords, bool relay, bool stopAtBracket) {
 
-  if (inWords[0] == '{')
+  if (!inWords.isEmpty() && inWords[0] == '{')
   {
     QByteArray retVal;
     int runLen = inWords.find ('}', 1);
@@ -2009,8 +2002,8 @@ bool imapParser::hasCapability (const QString & cap)
 {
   QString c = cap.toLower();
 //  kDebug(7116) <<"imapParser::hasCapability - Looking for '" << cap <<"'";
-  for (QStringList::ConstIterator it = imapCapabilities.begin ();
-       it != imapCapabilities.end (); ++it)
+  for (QStringList::ConstIterator it = imapCapabilities.constBegin ();
+       it != imapCapabilities.constEnd (); ++it)
   {
 //    kDebug(7116) <<"imapParser::hasCapability - Examining '" << (*it) <<"'";
     if ( !(kasciistricmp(c.toAscii(), (*it).toAscii())) )
@@ -2032,9 +2025,9 @@ QString imapParser::namespaceForBox( const QString & box )
   QString myNamespace;
   if ( !box.isEmpty() )
   {
-    QList<QString> list = namespaceToDelimiter.keys();
+    const QList<QString> list = namespaceToDelimiter.keys();
     QString cleanPrefix;
-    for ( QList<QString>::Iterator it = list.begin(); it != list.end(); ++it )
+    for ( QList<QString>::ConstIterator it = list.begin(); it != list.end(); ++it )
     {
       if ( !(*it).isEmpty() && box.contains( *it ) )
         return (*it);
