@@ -82,7 +82,7 @@ mimeHeader::addHdrLine (mimeHdrLine * aHdrLine)
     {
       int skip;
       const char *aCStr = addLine->getValue ().data ();
-      QHash < QString, QString > aList;
+      QHash < QString, QString > *aList = 0;
 
       skip = mimeHdrLine::parseSeparator (';', aCStr);
       if (skip > 0)
@@ -104,16 +104,15 @@ mimeHeader::addHdrLine (mimeHdrLine * aHdrLine)
 
         if (!qstricmp (addLine->getLabel (), "Content-Disposition"))
         {
-          aList = dispositionList;
+          aList = &dispositionList;
           setDisposition( mimeValue );
         }
         else if (!qstricmp (addLine->getLabel (), "Content-Type"))
         {
-          aList = typeList;
+          aList = &typeList;
           setType( mimeValue );
         }
-        else
-          if (!qstricmp (addLine->getLabel (), "Content-Transfer-Encoding"))
+        else if (!qstricmp (addLine->getLabel (), "Content-Transfer-Encoding"))
         {
           setEncoding( mimeValue );
         }
@@ -144,8 +143,8 @@ mimeHeader::addHdrLine (mimeHdrLine * aHdrLine)
         {
           if (skip > 0)
           {
-            addParameter (QByteArray (aCStr, skip).simplified(), aList);
-//            cout << "-- '" << aParm.data() << "'" << endl;
+            if (aList)
+              addParameter (QByteArray (aCStr, skip).simplified(), *aList);
             mimeValue = QByteArray (addLine->getValue ().data (), skip);
             aCStr += skip;
           }
@@ -542,7 +541,7 @@ bool mimeHeader::parseHeader (mimeIO & useIO)
   while (useIO.inputLine (inputStr))
   {
     int appended;
-    if (!inputStr.startsWith ("From ") || !first)
+    if (!inputStr.startsWith("From ") || !first) //krazy:exclude=strings
     {
       first = false;
       appended = my_line.appendStr (inputStr);
@@ -647,13 +646,13 @@ mimeHeader::bodyDecodedBinary ()
 {
   QByteArray retVal;
 
-  if (contentEncoding.startsWith ("quoted-printable", Qt::CaseInsensitive) )
+  if (contentEncoding.startsWith (QLatin1String("quoted-printable"), Qt::CaseInsensitive) )
     retVal = KCodecs::quotedPrintableDecode(postMultipartBody);
-  else if (contentEncoding.startsWith ("base64", Qt::CaseInsensitive) )
+  else if (contentEncoding.startsWith (QLatin1String("base64"), Qt::CaseInsensitive) )
     KCodecs::base64Decode(postMultipartBody, retVal);
   else retVal = postMultipartBody;
 
-  kDebug(7116) <<"mimeHeader::bodyDecodedBinary - size is" << retVal.size ();
+  kDebug(7116) << "mimeHeader::bodyDecodedBinary - size is" << retVal.size ();
   return retVal;
 }
 
@@ -669,9 +668,9 @@ mimeHeader::setBodyEncoded (const QByteArray & _arr)
   QByteArray setVal;
 
   kDebug(7116) <<"mimeHeader::setBodyEncoded - in size" << _arr.size ();
-  if (contentEncoding.startsWith ("quoted-printable", Qt::CaseInsensitive) )
+  if (contentEncoding.startsWith (QLatin1String("quoted-printable"), Qt::CaseInsensitive) )
     setVal = KCodecs::quotedPrintableEncode(_arr);
-  else if (contentEncoding.startsWith ("base64", Qt::CaseInsensitive) )
+  else if (contentEncoding.startsWith (QLatin1String("base64"), Qt::CaseInsensitive) )
     KCodecs::base64Encode(_arr, setVal);
   else
     setVal.duplicate (_arr);
