@@ -1,7 +1,7 @@
 /* This file is part of KDE
    Copyright (C) 2000 by Wolfram Diestel <wolfram@steloj.de>
    Copyright (C) 2005 by Tim Way <tim@way.hrcoxmail.com>
-   Copyright (C) 2005 by Volker Krause <volker.krause@rwth-aachen.de>
+   Copyright (C) 2005 by Volker Krause <vkrause@kde.org>
 
    This is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -23,8 +23,7 @@
 #include <kglobal.h>
 #include <klocale.h>
 
-#define NNTP_PORT 119
-#define NNTPS_PORT 563
+#include <kio/ioslave_defaults.h>
 
 #define DBG_AREA 7114
 #define DBG kDebug(DBG_AREA)
@@ -60,17 +59,18 @@ int kdemain(int argc, char **argv) {
 /****************** NNTPProtocol ************************/
 
 NNTPProtocol::NNTPProtocol ( const QByteArray & pool, const QByteArray & app, bool isSSL )
-  : TCPSlaveBase((isSSL ? "nntps" : "nntp"), pool, app, isSSL )
+  : TCPSlaveBase((isSSL ? "nntps" : "nntp"), pool, app, isSSL ),
+    isAuthenticated( false )
 {
-  DBG << "=============> NNTPProtocol::NNTPProtocol" << endl;
+  DBG << "=============> NNTPProtocol::NNTPProtocol";
 
   readBufferLen = 0;
-  m_defaultPort = isSSL ? NNTPS_PORT : NNTP_PORT;
+  m_defaultPort = isSSL ? DEFAULT_NNTPS_PORT : DEFAULT_NNTP_PORT;
   m_port = m_defaultPort;
 }
 
 NNTPProtocol::~NNTPProtocol() {
-  DBG << "<============= NNTPProtocol::~NNTPProtocol" << endl;
+  DBG << "<============= NNTPProtocol::~NNTPProtocol";
 
   // close connection
   nntp_close();
@@ -79,8 +79,8 @@ NNTPProtocol::~NNTPProtocol() {
 void NNTPProtocol::setHost ( const QString & host, quint16 port, const QString & user,
                              const QString & pass )
 {
-  DBG << "setHost: " << ( ! user.isEmpty() ? (user+'@') : QString(""))
-      << host << ":" << ( ( port == 0 ) ? m_defaultPort : port  ) << endl;
+  DBG << ( ! user.isEmpty() ? (user+'@') : QString(""))
+      << host << ":" << ( ( port == 0 ) ? m_defaultPort : port  );
 
   if ( isConnected() && (mHost != host || m_port != port ||
        mUser != user || mPass != pass) )
@@ -94,14 +94,13 @@ void NNTPProtocol::setHost ( const QString & host, quint16 port, const QString &
 
 void NNTPProtocol::get( const KUrl& url )
 {
-  DBG << "get " << url.prettyUrl() << endl;
+  DBG << url.prettyUrl();
   QString path = QDir::cleanPath(url.path());
-  path = KUrl::fromPercentEncoding( path.toLatin1() );
 
   // path should be like: /group/<msg_id> or /group/<serial number>
-  if ( path.startsWith( QDir::separator() ) )
+  if ( path.startsWith( '/' ) )
     path.remove( 0, 1 );
-  int pos = path.indexOf( QDir::separator() );
+  int pos = path.indexOf( '/' );
   QString group;
   QString msg_id;
   if ( pos > 0 ) {
@@ -115,7 +114,7 @@ void NNTPProtocol::get( const KUrl& url )
   }
 
   int res_code;
-  DBG << "get group: " << group << " msg: " << msg_id << endl;
+  DBG << "group:" << group << "msg:" << msg_id;
 
   if ( !nntp_open() )
     return;
@@ -148,34 +147,34 @@ void NNTPProtocol::get( const KUrl& url )
   }
 
   // read and send data
-  QByteArray buffer;
   char tmp[MAX_PACKET_LEN];
-  int len = 0;
   while ( true ) {
     if ( !waitForResponse( readTimeout() ) ) {
       error( ERR_SERVER_TIMEOUT, mHost );
+      nntp_close();
       return;
     }
-    memset( tmp, 0, MAX_PACKET_LEN );
-    len = readLine( tmp, MAX_PACKET_LEN );
-    buffer = QByteArray( tmp, len );
+    int len = readLine( tmp, MAX_PACKET_LEN );
+    const char* buffer = tmp;
     if ( len <= 0 )
       break;
-    if ( buffer == ".\r\n" )
+    if ( len == 3 && tmp[0] == '.' && tmp[1] == '\r' && tmp[2] == '\n')
       break;
-    if ( buffer.startsWith( ".." ) )
-      buffer.remove( 0, 1 );
-    data( buffer );
+    if ( len > 1 && tmp[0] == '.' && tmp[1] == '.' ) {
+      ++buffer;
+      --len;
+    }
+    data( QByteArray::fromRawData( buffer, len ) );
   }
+
   // end of data
-  buffer.resize(0);
-  data(buffer);
+  data(QByteArray());
 
   // finish
   finished();
 }
 
-void NNTPProtocol::put( const KUrl &/*url*/, int /*permissions*/, KIO::JobFlags flags )
+void NNTPProtocol::put( const KUrl &/*url*/, int /*permissions*/, KIO::JobFlags /*flags*/ )
 {
   if ( !nntp_open() )
     return;
@@ -200,7 +199,7 @@ void NNTPProtocol::special(const QByteArray& data) {
 }
 
 bool NNTPProtocol::post_article() {
-  DBG << "post article " << endl;
+  DBG;
 
   // send post command
   infoMessage( i18n("Sending article...") );
@@ -220,7 +219,7 @@ bool NNTPProtocol::post_article() {
     QByteArray buffer;
     dataReq();
     result = readData( buffer );
-    DBG << "receiving data: " << QString(buffer) << endl;
+    DBG << "receiving data:" << buffer;
     // treat the buffer data
     if ( result > 0 ) {
       // translate "\r\n." to "\r\n.."
@@ -229,7 +228,7 @@ bool NNTPProtocol::post_article() {
         buffer.insert( 0, '.' );
         pos += 2;
       }
-      last_chunk_had_line_ending = ( buffer.endsWith( "\r\n" ) );
+      last_chunk_had_line_ending = ( buffer.endsWith( "\r\n" ) ); //krazy:exclude=strings
       while ( (pos = buffer.indexOf( "\r\n.", pos )) > 0) {
         buffer.insert( pos + 2, '.' );
         pos += 4;
@@ -237,13 +236,13 @@ bool NNTPProtocol::post_article() {
 
       // send data to socket, write() doesn't send the terminating 0
       write( buffer, buffer.length() );
-      DBG << "writing: " << QString( buffer ) << endl;
+      DBG << "writing:" << buffer;
     }
   } while ( result > 0 );
 
   // error occurred?
   if (result<0) {
-    ERR << "error while getting article data for posting" << endl;
+    ERR << "error while getting article data for posting";
     nntp_close();
     return false;
   }
@@ -266,7 +265,7 @@ bool NNTPProtocol::post_article() {
 
 
 void NNTPProtocol::stat( const KUrl& url ) {
-  DBG << "stat " << url.prettyUrl() << endl;
+  DBG << url.prettyUrl();
   UDSEntry entry;
   QString path = QDir::cleanPath(url.path());
   QRegExp regGroup = QRegExp("^\\/?[a-z0-9\\.\\-_]+\\/?$",Qt::CaseInsensitive);
@@ -277,7 +276,7 @@ void NNTPProtocol::stat( const KUrl& url ) {
 
   // / = group list
   if (path.isEmpty() || path == "/") {
-    DBG << "stat root" << endl;
+    DBG << "root";
     fillUDSEntry( entry, QString(), 0, false, ( S_IWUSR | S_IWGRP | S_IWOTH ) );
 
   // /group = message list
@@ -285,7 +284,7 @@ void NNTPProtocol::stat( const KUrl& url ) {
     if ( path.startsWith( '/' ) ) path.remove(0,1);
     if ((pos = path.indexOf('/')) > 0) group = path.left(pos);
     else group = path;
-    DBG << "stat group: " << group << endl;
+    DBG << "group:" << group;
     // postingAllowed should be ored here with "group not moderated" flag
     // as size the num of messages (GROUP cmd) could be given
     fillUDSEntry( entry, group, 0, false, ( S_IWUSR | S_IWGRP | S_IWOTH ) );
@@ -298,7 +297,7 @@ void NNTPProtocol::stat( const KUrl& url ) {
     if ( group.startsWith( '/' ) )
       group.remove( 0, 1 );
     if ((pos = group.indexOf('/')) > 0) group = group.left(pos);
-    DBG << "stat group: " << group << " msg: " << msg_id << endl;
+    DBG << "group:" << group << "msg:" << msg_id;
     fillUDSEntry( entry, msg_id, 0, true );
 
   // invalid url
@@ -312,7 +311,7 @@ void NNTPProtocol::stat( const KUrl& url ) {
 }
 
 void NNTPProtocol::listDir( const KUrl& url ) {
-  DBG << "listDir " << url.prettyUrl() << endl;
+  DBG << url.prettyUrl();
   if ( !nntp_open() )
     return;
 
@@ -322,7 +321,7 @@ void NNTPProtocol::listDir( const KUrl& url ) {
   {
     KUrl newURL(url);
     newURL.setPath("/");
-    DBG << "listDir redirecting to " << newURL.prettyUrl() << endl;
+    DBG << "redirecting to" << newURL.prettyUrl();
     redirection(newURL);
     finished();
     return;
@@ -380,9 +379,9 @@ void NNTPProtocol::fetchGroups( const QString &since, bool desc )
   while ( true ) {
     if ( ! waitForResponse( readTimeout() ) ) {
       error( ERR_SERVER_TIMEOUT, mHost );
+      nntp_close();
       return;
     }
-    memset( readBuffer, 0, MAX_PACKET_LEN );
     readBufferLen = readLine ( readBuffer, MAX_PACKET_LEN );
     line = QByteArray( readBuffer, readBufferLen );
     if ( line == ".\r\n" )
@@ -451,15 +450,15 @@ void NNTPProtocol::fetchGroups( const QString &since, bool desc )
     while ( true ) {
       if ( ! waitForResponse( readTimeout() ) ) {
         error( ERR_SERVER_TIMEOUT, mHost );
+        nntp_close();
         return;
       }
-      memset( readBuffer, 0, MAX_PACKET_LEN );
       readBufferLen = readLine ( readBuffer, MAX_PACKET_LEN );
       line = QByteArray( readBuffer, readBufferLen );
       if ( line == ".\r\n" )
         break;
 
-      //DBG << "  fetching group description: " << QString( line ).trimmed() << endl;
+      //DBG << "  fetching group description: " << QString( line ).trimmed();
       int pos = line.indexOf( ' ' );
       pos = pos < 0 ? line.indexOf( '\t' ) : qMin( pos, line.indexOf( '\t' ) );
       group = line.left( pos );
@@ -518,10 +517,15 @@ bool NNTPProtocol::fetchGroup( QString &group, unsigned long first, unsigned lon
   if (firstSerNum == 0)
     return true;
   first = qMax( first, firstSerNum );
+  if ( lastSerNum < first ) { // No need to fetch anything
+    // note: this also ensure that "lastSerNum - first" is not negative
+    // in the next test (in "unsigned long" computation this leads to an overflow
+    return true;
+  }
   if ( max > 0 && lastSerNum - first > max )
     first = lastSerNum - max + 1;
 
-  DBG << "Starting from serial number: " << first << " of " << firstSerNum << " - " << lastSerNum << endl;
+  DBG << "Starting from serial number: " << first << " of " << firstSerNum << " - " << lastSerNum;
   setMetaData( "FirstSerialNumber", QString::number( firstSerNum ) );
   setMetaData( "LastSerialNumber", QString::number( lastSerNum ) );
 
@@ -603,15 +607,15 @@ bool NNTPProtocol::fetchGroupXOVER( unsigned long first, bool &notSupported )
     while ( true ) {
       if ( ! waitForResponse( readTimeout() ) ) {
         error( ERR_SERVER_TIMEOUT, mHost );
+        nntp_close();
         return false;
       }
-      memset( readBuffer, 0, MAX_PACKET_LEN );
       readBufferLen = readLine ( readBuffer, MAX_PACKET_LEN );
       line = QString::fromLatin1( readBuffer, readBufferLen );
       if ( line == ".\r\n" )
         break;
       headers << line.trimmed();
-      DBG << "OVERVIEW.FMT: " << line.trimmed() << endl;
+      DBG << "OVERVIEW.FMT:" << line.trimmed();
     }
   } else {
     // fallback to defaults
@@ -624,8 +628,10 @@ bool NNTPProtocol::fetchGroupXOVER( unsigned long first, bool &notSupported )
     return true; // no articles selected
   if ( res == 500 )
     notSupported = true; // unknwon command
-  if ( res != 224 )
+  if ( res != 224 ) {
+    unexpected_response( res, "XOVER" );
     return false;
+  }
 
   long msgSize;
   QString name;
@@ -636,9 +642,9 @@ bool NNTPProtocol::fetchGroupXOVER( unsigned long first, bool &notSupported )
   while ( true ) {
     if ( ! waitForResponse( readTimeout() ) ) {
       error( ERR_SERVER_TIMEOUT, mHost );
+      nntp_close();
       return false;
     }
-    memset( readBuffer, 0, MAX_PACKET_LEN );
     readBufferLen = readLine ( readBuffer, MAX_PACKET_LEN );
     line = QString::fromLatin1( readBuffer, readBufferLen );
     if ( line == ".\r\n" ) {
@@ -647,7 +653,7 @@ bool NNTPProtocol::fetchGroupXOVER( unsigned long first, bool &notSupported )
       return true;
     }
 
-    fields = line.split( "\t", QString::KeepEmptyParts);
+    fields = line.split( '\t', QString::KeepEmptyParts);
     msgSize = 0;
     entry.clear();
     udsType = KIO::UDSEntry::UDS_EXTRA;
@@ -662,7 +668,7 @@ bool NNTPProtocol::fetchGroupXOVER( unsigned long first, bool &notSupported )
         continue;
       }
       QString atomStr;
-      if ( (*it).endsWith( "full" ) )
+      if ( (*it).endsWith( QLatin1String( "full" ) ) )
         if ( (*it2).trimmed().isEmpty() )
           atomStr = (*it).left( (*it).indexOf( ':' ) + 1 ); // strip of the 'full' suffix
         else
@@ -716,7 +722,7 @@ void NNTPProtocol::nntp_close () {
   if ( isConnected() ) {
     write( "QUIT\r\n", 6 );
     disconnectFromHost();
-    opened = false;
+    isAuthenticated = false;
   }
   mCurrentGroup.clear();
 }
@@ -725,16 +731,16 @@ bool NNTPProtocol::nntp_open()
 {
   // if still connected reuse connection
   if ( isConnected() ) {
-    DBG << "reusing old connection" << endl;
+    DBG << "reusing old connection";
     return true;
   }
 
-  DBG << "  nntp_open -- creating a new connection to " << mHost << ":" << m_port << endl;
+  DBG << "  nntp_open -- creating a new connection to" << mHost << ":" << m_port;
   // create a new connection (connectToHost() includes error handling)
   infoMessage( i18n("Connecting to server...") );
-  if ( connectToHost( (isAutoSsl() ? "nntps" : "nntp"), mHost.toLatin1(), m_port ) )
+  if ( connectToHost( (isAutoSsl() ? "nntps" : "nntp"), mHost, m_port ) )
   {
-    DBG << "  nntp_open -- connection is open " << endl;
+    DBG << "  nntp_open -- connection is open";
 
     // read greeting
     int res_code = evalResponse( readBuffer, readBufferLen );
@@ -749,9 +755,7 @@ bool NNTPProtocol::nntp_open()
       return false;
     }
 
-    DBG << "  nntp_open -- greating was read res_code : " << res_code << endl;
-    // let local class know that we are connected
-    opened = true;
+    DBG << "  nntp_open -- greating was read res_code :" << res_code;
 
     res_code = sendCommand("MODE READER");
 
@@ -776,6 +780,9 @@ bool NNTPProtocol::nntp_open()
       }
     }
 
+    // *try* to authenticate now (see bug#167718)
+    authenticate();
+
     return true;
   }
 
@@ -786,22 +793,22 @@ int NNTPProtocol::sendCommand( const QString &cmd )
 {
   int res_code = 0;
 
-  if ( !opened ) {
-    ERR << "NOT CONNECTED, cannot send cmd " << cmd << endl;
+  if ( !nntp_open() ) {
+    ERR << "NOT CONNECTED, cannot send cmd" << cmd;
     return 0;
   }
 
-  DBG << "sending cmd " << cmd << endl;
+  DBG << "cmd:" << cmd;
 
   write( cmd.toLatin1(), cmd.length() );
   // check the command for proper termination
-  if ( !cmd.endsWith( "\r\n" ) )
+  if ( !cmd.endsWith( QLatin1String( "\r\n" ) ) )
     write( "\r\n", 2 );
   res_code =  evalResponse( readBuffer, readBufferLen );
 
   // if authorization needed send user info
   if (res_code == 480) {
-    DBG << "auth needed, sending user info" << endl;
+    DBG << "auth needed, sending user info";
 
     if ( mUser.isEmpty() || mPass.isEmpty() ) {
       KIO::AuthInfo authInfo;
@@ -815,23 +822,7 @@ int NNTPProtocol::sendCommand( const QString &cmd )
     if ( mUser.isEmpty() || mPass.isEmpty() )
       return res_code;
 
-    // send username to server and confirm response
-    write( "AUTHINFO USER ", 14 );
-    write( mUser.toLatin1(), mUser.length() );
-    write( "\r\n", 2 );
-    res_code = evalResponse( readBuffer, readBufferLen );
-
-    if (res_code != 381) {
-      // error should be handled by invoking function
-      return res_code;
-    }
-
-    // send password
-    write( "AUTHINFO PASS ", 14 );
-    write( mPass.toLatin1(), mPass.length() );
-    write( "\r\n", 2 );
-    res_code = evalResponse( readBuffer, readBufferLen );
-
+    res_code = authenticate();
     if (res_code != 281) {
       // error should be handled by invoking function
       return res_code;
@@ -839,7 +830,7 @@ int NNTPProtocol::sendCommand( const QString &cmd )
 
     // ok now, resend command
     write( cmd.toLatin1(), cmd.length() );
-    if ( !cmd.endsWith( "\r\n" ) )
+    if ( !cmd.endsWith( QLatin1String( "\r\n" ) ) )
       write( "\r\n", 2 );
     res_code = evalResponse( readBuffer, readBufferLen );
   }
@@ -847,19 +838,74 @@ int NNTPProtocol::sendCommand( const QString &cmd )
   return res_code;
 }
 
-void NNTPProtocol::unexpected_response( int res_code, const QString &command )
+int NNTPProtocol::authenticate()
 {
-  ERR << "Unexpected response to " << command << " command: (" << res_code << ") "
-      << readBuffer << endl;
+  int res_code = 0;
 
-  KIO::Error errCode;
-  switch ( res_code ) {
-    case 480: errCode = ERR_COULD_NOT_LOGIN; break;
-    default: errCode = ERR_INTERNAL;
+  if( isAuthenticated ) {
+    // already authenticated
+    return 281;
   }
 
-  error( errCode, i18n("Unexpected server response to %1 command:\n%2",
-     command, readBuffer ) );
+  if( mUser.isEmpty() || mPass.isEmpty() ) {
+    return 281; // failsafe : maybe add a "relax" mode to optionally ask user/pwd.
+  }
+
+  // send username to server and confirm response
+  write( "AUTHINFO USER ", 14 );
+  write( mUser.toLatin1(), mUser.length() );
+  write( "\r\n", 2 );
+  res_code = evalResponse( readBuffer, readBufferLen );
+
+  if( res_code == 281 ) {
+    // no password needed (RFC 2980 3.1.1 does not required one)
+    return res_code;
+  }
+  if (res_code != 381) {
+    // error should be handled by invoking function
+    return res_code;
+  }
+
+  // send password
+  write( "AUTHINFO PASS ", 14 );
+  write( mPass.toLatin1(), mPass.length() );
+  write( "\r\n", 2 );
+  res_code = evalResponse( readBuffer, readBufferLen );
+
+  if( res_code == 281 ) {
+    isAuthenticated = true;
+  }
+
+  return res_code;
+}
+
+void NNTPProtocol::unexpected_response( int res_code, const QString &command )
+{
+  ERR << "Unexpected response to" << command << "command: (" << res_code << ")"
+      << readBuffer;
+
+  // See RFC 3977 appendix C "Summary of Response Codes"
+  switch ( res_code ) {
+    case 205: // connection closed by the server: this can happens, e.g. if the session timeout on the server side
+      // Not the same thing, but use the same message as code 400 anyway.
+    case 400: // temporary issue on the server
+      error( ERR_INTERNAL_SERVER,
+             i18n( "The server %1 could not handle your request.\n"
+                   "Please try again now, or later if the problem persists.", mHost ) );
+      break;
+    case 480: // credential request
+      error( ERR_COULD_NOT_LOGIN,
+             i18n( "You need to authenticate to access the requested resource." ) );
+    case 481: // wrong credential (TODO: place a specific message for this case)
+      error( ERR_COULD_NOT_LOGIN,
+             i18n( "The supplied login and/or password are incorrect." ) );
+      break;
+    case 502:
+      error( ERR_ACCESS_DENIED, mHost );
+      break;
+    default:
+      error( ERR_INTERNAL, i18n( "Unexpected server response to %1 command:\n%2", command, readBuffer ) );
+  }
 
   nntp_close();
 }
@@ -868,9 +914,9 @@ int NNTPProtocol::evalResponse ( char *data, ssize_t &len )
 {
   if ( !waitForResponse( responseTimeout() ) ) {
     error( ERR_SERVER_TIMEOUT , mHost );
+    nntp_close();
     return -1;
   }
-  memset( data, 0, MAX_PACKET_LEN );
   len = readLine( data, MAX_PACKET_LEN );
 
   if ( len < 3 )
@@ -879,7 +925,7 @@ int NNTPProtocol::evalResponse ( char *data, ssize_t &len )
   // get the first three characters. should be the response code
   int respCode = ( ( data[0] - 48 ) * 100 ) + ( ( data[1] - 48 ) * 10 ) + ( ( data[2] - 48 ) );
 
-  DBG << "evalResponse - got: " << respCode << endl;
+  DBG << "got:" << respCode;
 
   return respCode;
 }
