@@ -178,14 +178,14 @@ void KLdapTest::testLdapConnection()
   LdapConnection conn;
   conn.setUrl( url );
   int ret;
-  if ( (ret = conn.connect()) ) {
+  if ( ( ret = conn.connect() ) ) {
     kDebug() << "Could not connect to LDAP server. Error was:" << conn.connectionError();
   }
   QCOMPARE( ret, 0 );
 
   LdapOperation op( conn );
   // Now attempt to bind
-  if ( (ret = op.bind_s()) ) {
+  if ( ( ret = op.bind_s() ) ) {
     kDebug() << "Could not bind to server. Error was:" << conn.ldapErrorString();
   }
   QCOMPARE( ret, 0 );
@@ -213,7 +213,7 @@ void KLdapTest::testLdapSearch()
 
 void KLdapTest::searchResult( KLDAP::LdapSearch *search )
 {
-  kDebug() << "KLdapTest::searchResult()";
+  kDebug();
   int err = search->error();
   if ( err ) {
     kDebug() << "Search returned the following error:" << search->errorString();
@@ -224,7 +224,7 @@ void KLdapTest::searchResult( KLDAP::LdapSearch *search )
 void KLdapTest::searchData( KLDAP::LdapSearch *search, const KLDAP::LdapObject &obj )
 {
   Q_UNUSED( search );
-  //kDebug() << "KLdapTest::searchData()";
+  //kDebug();
   //kDebug() << "Object:";
   //kDebug() << obj.toString();
   m_objects.append( obj );
@@ -248,13 +248,13 @@ void KLdapTest::testLdapModel()
   LdapConnection conn;
   conn.setUrl( url );
   int ret;
-  if ( (ret = conn.connect()) ) {
+  if ( ( ret = conn.connect() ) ) {
     kDebug() << "Could not connect to LDAP server. Error was:" << conn.connectionError();
   }
   QCOMPARE( ret, 0 );
 
   LdapOperation op( conn );
-  if ( (ret = op.bind_s()) ) {
+  if ( ( ret = op.bind_s() ) ) {
     kDebug() << "Could not bind to server. Error was:" << conn.ldapErrorString();
   }
   QCOMPARE( ret, 0 );
