@@ -1,7 +1,7 @@
 /*  -*- c++ -*-
     kmime_header_parsing.h
 
-    KMime, the KDE internet mail/usenet news message library.
+    KMime, the KDE Internet mail/usenet news message library.
     Copyright (c) 2001-2002 Marc Mutz <mutz@kde.org>
 
     This library is free software; you can redistribute it and/or
@@ -35,6 +35,10 @@ class QStringList;
 
 namespace KMime {
 
+namespace Headers {
+  class Base;
+}
+
 namespace Types {
 
 // for when we can't make up our mind what to use...
@@ -46,6 +50,8 @@ struct KMIME_EXPORT QStringOrQPair {
 
 struct KMIME_EXPORT AddrSpec {
   QString asString() const;
+  /*! This is the same as asString(), except it decodes IDNs for display */
+  QString asPrettyString() const;
   bool isEmpty() const;
   QString localPart;
   QString domain;
@@ -325,6 +331,14 @@ KMIME_EXPORT bool parseTime( const char* &scursor, const char * const send,
 KMIME_EXPORT bool parseDateTime( const char* &scursor, const char * const send,
                                  KDateTime &result, bool isCRLF=false );
 
+/**
+ * Extracts and returns the first header that is contained in the given byte array.
+ * The header will also be removed from the passed-in byte array head.
+ *
+ * @since 4.4
+ */
+KMIME_EXPORT KMime::Headers::Base *extractFirstHeader( QByteArray &head );
+
 } // namespace HeaderParsing
 
 } // namespace KMime
