@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2006 Volker Krause <volker.krause@rwth-aachen.de>
+    Copyright (c) 2006 Volker Krause <vkrause@kde.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -19,7 +19,7 @@
 #ifndef KMIME_UTIL_TEST_H
 #define KMIME_UTIL_TEST_H
 
-#include <QObject>
+#include <QtCore/QObject>
 
 class KMimeUtilTest : public QObject
 {
@@ -27,6 +27,8 @@ class KMimeUtilTest : public QObject
   private Q_SLOTS:
     void testUnfoldHeader();
     void testExtractHeader();
+    void testBalanceBidiState();
+    void testBalanceBidiState_data();
 };
 
 #endif
