@@ -268,9 +268,9 @@ void LinkLocatorTest::testHtmlConvert_data()
   QTest::addColumn<int>("flags");
   QTest::addColumn<QString>("htmlText");
 
-  //QTest::newRow( "" ) << "foo" << 0 << "foo";
-  //QTest::newRow( "" ) << "  foo " << 0 << "  foo ";
-  // Linker error when using PreserveSpaces, therefore the hardcoded 0x01
+  // Linker error when using PreserveSpaces, therefore the hardcoded 0x01 or 0x09
+
+  // Test preserving whitespace correctly
   QTest::newRow( "" ) << " foo" << 0x01 << "&nbsp;foo";
   QTest::newRow( "" ) << "  foo" << 0x01 << "&nbsp;&nbsp;foo";
   QTest::newRow( "" ) << "  foo  " << 0x01 << "&nbsp;&nbsp;foo&nbsp;&nbsp;";
@@ -283,9 +283,37 @@ void LinkLocatorTest::testHtmlConvert_data()
   QTest::newRow( "" ) << " bla bla \n bla bla a\n  bla bla " << 0x01
                       << "&nbsp;bla bla&nbsp;<br />\n&nbsp;bla bla a<br />\n&nbsp;&nbsp;bla bla&nbsp;";
 
+  // Test highlighting with *, / and _
+  QTest::newRow( "" ) << "Ce paragraphe _contient_ des mots ou des _groupes de mots_ à mettre en forme…" << 0x09 << "Ce paragraphe <u>contient</u> des mots ou des <u>groupes de mots</u> à mettre en forme…";
+  QTest::newRow( "punctation-bug" ) << "Ce texte *a l'air* de _fonctionner_, à condition d’utiliser le guillemet ASCII." << 0x09 << "Ce texte <b>a l'air</b> de <u>fonctionner</u>, à condition d’utiliser le guillemet ASCII.";
+  QTest::newRow( "punctation-bug" ) << "Un répertoire /est/ un *dossier* où on peut mettre des *fichiers*." << 0x09 << "Un répertoire <i>est</i> un <b>dossier</b> où on peut mettre des <b>fichiers</b>.";
+  QTest::newRow( "punctation-bug" ) << "*BLA BLA BLA BLA*." << 0x09 << "<b>BLA BLA BLA BLA</b>.";
+  QTest::newRow( "" ) << "Je vais tenter de repérer des faux positif*" << 0x09 << "Je vais tenter de repérer des faux positif*";
+  QTest::newRow( "" ) << "*Ouais !* *Yes!*" << 0x09 << "<b>Ouais !</b> <b>Yes!</b>";
+  QTest::newRow( "" ) << "the /etc/{rsyslog.d,syslog-ng.d}/package.rpmnew file" << 0x09 << "the /etc/{rsyslog.d,syslog-ng.d}/package.rpmnew file";
+
+  // This test has problems with the encoding, apparently.
+  //QTest::newRow( "" ) << "*Ça fait plaisir de pouvoir utiliser des lettres accentuées dans du texte mis en forme*." << 0x09 << "<b>Ça fait plaisir de pouvoir utiliser des lettres accentuées dans du texte mis en forme</b>.";
+
   // Bug reported by dfaure, the <hostname> would get lost
   QTest::newRow( "" ) << "KUrl url(\"http://strange<hostname>/\");" << (0x08 | 0x02)
                       << "KUrl url(&quot;<a href=\"http://strange<hostname>/\">http://strange&lt;hostname&gt;/</a>&quot;);";
+
+  // Bug: 211128 - plain text emails should not replace ampersand & with &amp;
+  QTest::newRow( "bug211128" ) << "https://green-site/?Ticket=85&Page=next" << 0x01
+    << "<a href=\"https://green-site/?Ticket=85&Page=next\">https://green-site/?Ticket=85&amp;Page=next</a>";
+
+  QTest::newRow( "dotBeforeEnd" ) << "Look at this file: www.example.com/example.h" << 0x01
+                                  << "Look at this file: <a href=\"http://www.example.com/example.h\">"
+                                     "www.example.com/example.h</a>";
+  QTest::newRow( "dotInMiddle" ) << "Look at this file: www.example.com/.bashrc" << 0x01
+                                 << "Look at this file: <a href=\"http://www.example.com/.bashrc\">"
+                                     "www.example.com/.bashrc</a>";
+
+  // A dot at the end of an URL is explicitly ignored
+  QTest::newRow( "dotAtEnd" ) << "Look at this file: www.example.com/test.cpp." << 0x01
+                              << "Look at this file: <a href=\"http://www.example.com/test.cpp\">"
+                                 "www.example.com/test.cpp</a>.";
 }
 
 void LinkLocatorTest::testHtmlConvert()
@@ -294,6 +322,8 @@ void LinkLocatorTest::testHtmlConvert()
   QFETCH(int, flags);
   QFETCH(QString, htmlText);
 
+  QEXPECT_FAIL( "punctation-bug", "Linklocator does not properly detect punctation as boundaries", Continue );
+
   QString actualHtml = LinkLocator::convertToHtml( plainText, flags );
   QCOMPARE( actualHtml, htmlText );
 }
