@@ -1,7 +1,7 @@
 /*
  * This file is part of the syndication library
  *
- * Copyright (C) 2006 Frank Osterfeld <frank.osterfeld@kdemail.net>
+ * Copyright (C) 2006 Frank Osterfeld <osterfeld@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -19,14 +19,14 @@
  * Boston, MA 02110-1301, USA.
  *
  */
-#include "constants.h"
 #include "elementwrapper.h"
+#include "constants.h"
 
 #include <kurl.h>
 
-#include <QDomElement>
-#include <QString>
-#include <QTextStream>
+#include <QtXml/QDomElement>
+#include <QtCore/QString>
+#include <QtCore/QTextStream>
 
 namespace Syndication {
 
