@@ -1,7 +1,7 @@
 /*
  * This file is part of the syndication library
  *
- * Copyright (C) 2006 Frank Osterfeld <frank.osterfeld@kdemail.net>
+ * Copyright (C) 2006 Frank Osterfeld <osterfeld@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -29,7 +29,7 @@
 namespace Syndication {
     
 class ImageRSS2Impl;
-typedef SharedPtr<ImageRSS2Impl> ImageRSS2ImplPtr;
+typedef boost::shared_ptr<ImageRSS2Impl> ImageRSS2ImplPtr;
 
 /**
  *
