@@ -1,7 +1,7 @@
 /*
  * This file is part of the syndication library
  *
- * Copyright (C) 2005 Frank Osterfeld <frank.osterfeld@kdemail.net>
+ * Copyright (C) 2005 Frank Osterfeld <osterfeld@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -24,7 +24,7 @@
 
 #include <syndication/rdf/document.h>
 #include <syndication/abstractparser.h>
-#include <syndication/sharedptr.h>
+#include <boost/shared_ptr.hpp>
 
 namespace Syndication {
 
@@ -32,7 +32,6 @@ class DocumentSource;
 
 namespace RDF {
 
-class Model;
 
 /**
  * Parser implementation for RDF-based RSS 0.9 and RSS 1.0 feeds.
@@ -80,7 +79,7 @@ class SYNDICATION_EXPORT Parser : public Syndication::AbstractParser
         Parser& operator=(const Parser& other);
 
         class ParserPrivate;
-        ParserPrivate* d;
+        ParserPrivate* const d;
 };
 
 } // namespace RDF
